/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2005 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "ButtonIcon.h"

ButtonIcon::ButtonIcon(const string &name)
{
	defaulticon.iconSize = GetxRCInt(name, "IconSize", 16, 8, 64);
	defaulticon.iconX = GetxRCString(name, "IconX", "0");
	defaulticon.iconY = GetxRCString(name, "IconY", "" );
	
	defaulticon.icon_saturation = GetxRCInt(name, "IconSaturationIntensity", 255, 0, 255);
	defaulticon.icon_hueintensity = GetxRCInt(name, "IconHueIntensity", 0, 0, 255);
	defaulticon.icon_huecolor = GetxRCColor(name, "IconHueColor", RGB(255, 255, 255));

	specialHover = GetxRCBool(name, "UseHoverIconSettings");
	specialGrouped = GetxRCBool(name, "UseGroupedIconSettings");
}

ButtonIcon::~ButtonIcon()
{
}

void ButtonIcon::subconfigure(string type, ICONSETTINGS &workSettings, ICONSETTINGS &defaultSettings)
{
	workSettings.iconSize = GetxRCInt(type, "IconSize", defaultSettings.iconSize, 8, 64);
	workSettings.iconX = GetxRCString(type, "IconX", defaultSettings.iconX);
	if ( defaulticon.iconY.empty() )
		workSettings.iconY = GetxRCString(type, "IconY", ("-" + IntToString(workSettings.iconSize/2) + "c"));
	else
		workSettings.iconY = GetxRCString(type, "IconY", defaulticon.iconY);

	workSettings.icon_saturation = GetxRCInt(type, "IconSaturationIntensity", defaultSettings.icon_saturation, 0, 255);
	workSettings.icon_hueintensity = GetxRCInt(type, "IconHueIntensity", defaultSettings.icon_hueintensity, 0, 255);
	workSettings.icon_huecolor = GetxRCColor(type, "IconHueColor", defaultSettings.icon_huecolor);
}

void ButtonIcon::configure(const string &name)
{
	subconfigure((name + "Normal"), normal, defaulticon);

	subconfigure((name + "Active"), active, normal);

	subconfigure((name + "Minimized"), minimized, normal);

	subconfigure((name + "Flashing"), flashing, normal);

	subconfigure((name + "Hover"), hover, defaulticon);

	subconfigure((name + "Grouped"), group, defaulticon);
}

void ButtonIcon::apply(HDC hDC, int x, int y, int width, int height, int buttonType, int buttonType2, HICON hIcon, bool useAlphaMap)
{
	if(hIcon == NULL)
		return;

	int iconX = 0;
	int iconY = 0;
	int _iconSize = 16;

	int _saturation = 255;
	int _hueIntensity = 0;
	COLORREF _hueColor = RGB(255, 255, 255);

	if (buttonType == 4)
	{
		if ( !specialHover )
		{
			buttonType = buttonType2;
		}
		else
		{
			iconX = xParseCoord(hover.iconX, 0, width, GetLitestepDesktop()); //GetLitestepDesktop as placeholder
			iconY = xParseCoord(hover.iconY, 0, height, GetLitestepDesktop()); //GetLitestepDesktop as placeholder

			_iconSize = hover.iconSize;
			_saturation = hover.icon_saturation;
			_hueIntensity = hover.icon_hueintensity;
			_hueColor = hover.icon_huecolor;
		}
	}
	else if (buttonType == 5)
	{
		if ( !specialGrouped )
		{
			buttonType = buttonType2;
		}
		else
		{
			iconX = xParseCoord(group.iconX, 0, width, GetLitestepDesktop()); //GetLitestepDesktop as placeholder
			iconY = xParseCoord(group.iconY, 0, height, GetLitestepDesktop()); //GetLitestepDesktop as placeholder

			_iconSize = group.iconSize;
			_saturation = group.icon_saturation;
			_hueIntensity = group.icon_hueintensity;
			_hueColor = group.icon_huecolor;
		}
	}

	if (buttonType == 0)
	{
		iconX = xParseCoord(normal.iconX, 0, width, GetLitestepDesktop()); //GetLitestepDesktop as placeholder
		iconY = xParseCoord(normal.iconY, 0, height, GetLitestepDesktop()); //GetLitestepDesktop as placeholder

		_iconSize = normal.iconSize;
		_saturation = normal.icon_saturation;
		_hueIntensity = normal.icon_hueintensity;
		_hueColor = normal.icon_huecolor;
	}
	else if (buttonType == 1)
	{
		iconX = xParseCoord(active.iconX, 0, width, GetLitestepDesktop()); //GetLitestepDesktop as placeholder
		iconY = xParseCoord(active.iconY, 0, height, GetLitestepDesktop()); //GetLitestepDesktop as placeholder

		_iconSize = active.iconSize;
		_saturation = active.icon_saturation;
		_hueIntensity = active.icon_hueintensity;
		_hueColor = active.icon_huecolor;
	}
	else if (buttonType == 2)
	{
		iconX = xParseCoord(minimized.iconX, 0, width, GetLitestepDesktop()); //GetLitestepDesktop as placeholder
		iconY = xParseCoord(minimized.iconY, 0, height, GetLitestepDesktop()); //GetLitestepDesktop as placeholder

		_iconSize = minimized.iconSize;
		_saturation = minimized.icon_saturation;
		_hueIntensity = minimized.icon_hueintensity;
		_hueColor = minimized.icon_huecolor;
	}
	else if (buttonType == 3)
	{
		iconX = xParseCoord(flashing.iconX, 0, width, GetLitestepDesktop()); //GetLitestepDesktop as placeholder
		iconY = xParseCoord(flashing.iconY, 0, height, GetLitestepDesktop()); //GetLitestepDesktop as placeholder

		_iconSize = flashing.iconSize;
		_saturation = flashing.icon_saturation;
		_hueIntensity = flashing.icon_hueintensity;
		_hueColor = flashing.icon_huecolor;
	}

	if ( (_saturation != 255 || _hueIntensity != 0 || useAlphaMap) && IsOS( OS_2KXP ) )
	{	
		BITMAPINFO alpha;
		ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
		alpha.bmiHeader.biWidth=_iconSize;      // Set size you need
		alpha.bmiHeader.biHeight=_iconSize;    // Set size you need
		alpha.bmiHeader.biPlanes=1;
		alpha.bmiHeader.biBitCount=32;      // Can be 8, 16, 32 bpp or other number
		alpha.bmiHeader.biSizeImage=0;
		alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
		alpha.bmiHeader.biClrUsed= 0;
		alpha.bmiHeader.biClrImportant= 0;
		VOID *pvBits;
		
		HDC BGDC = CreateCompatibleDC(NULL);

		HBITMAP	BGBitmap = CreateDIBSection( BGDC,
												&alpha,
												DIB_RGB_COLORS,
												&pvBits,
												NULL,
												0 );
		HBITMAP oldBGBitmap = (HBITMAP) SelectObject(BGDC, BGBitmap);

		//Copy RealBG in Result Background
		BitBlt( BGDC, 0, 0, _iconSize, _iconSize, hDC, x+iconX, y+iconY, SRCCOPY );
	
		HDC bufferDC = CreateCompatibleDC(NULL);

		HBITMAP	bufferBitmap = CreateDIBSection( bufferDC,
													&alpha,
													DIB_RGB_COLORS,
													&pvBits,
													NULL,
													0 );

		HBITMAP oldbufferBitmap = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

		BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, 255, AC_SRC_ALPHA };

		AlphaBlend(bufferDC, 0, 0, _iconSize, _iconSize, hDC, x+iconX, y+iconY, _iconSize, _iconSize, blendFunction);

		DrawIconEx(bufferDC, 0, 0, hIcon, _iconSize, _iconSize, 0, NULL, DI_NORMAL);

		HDC maskDC = CreateCompatibleDC(NULL);

		HBITMAP maskBitmap = CreateDIBSection( maskDC,
												&alpha,
												DIB_RGB_COLORS,
												&pvBits,
												NULL,
												0 );

		HBITMAP oldmaskBitmap = (HBITMAP) SelectObject(maskDC, maskBitmap);

		DrawIconEx(maskDC, 0, 0, hIcon, _iconSize, _iconSize, 0, NULL, DI_MASK);

		BITMAP bm_b;
		GetObject(bufferBitmap, sizeof(BITMAP), &bm_b);
		BYTE *pPixel_b = (LPBYTE)bm_b.bmBits;

		BITMAP bm_m;
		GetObject(maskBitmap, sizeof(BITMAP), &bm_m);
		BYTE *pPixel_m = (LPBYTE)bm_m.bmBits;

		BITMAP bm_bg;
		GetObject(BGBitmap, sizeof(BITMAP), &bm_bg);
		BYTE *pPixel_bg = (LPBYTE)bm_bg.bmBits;

		BYTE r = 0;
		BYTE g = 0;
		BYTE b = 0;
		bool skip = false;
	
		for ( int j = 0; j < bm_b.bmHeight * bm_b.bmWidth; ++j )
		{		
			//Filter Mask and insert OriginalBG to preserve AlphaChannel
			if ( pPixel_m[0] == 255 && pPixel_m[1] == 255 && pPixel_m[2] == 255 )
			{
				pPixel_b[0] = 0;	
				pPixel_b[1] = 0;	
				pPixel_b[2] = 0;
				pPixel_b[3] = 0;

				skip = true;
			}
			//For colorizing without AlphaMap
			if ( !skip && pPixel_bg[0] == 255 && pPixel_bg[1] == 0 && pPixel_bg[2] == 255 && pPixel_b[3] != 255 && pPixel_b[3] != 0 )
			{
				pPixel_b[0] = 0;	
				pPixel_b[1] = 0;	
				pPixel_b[2] = 0;	
				pPixel_b[3] = 0;

				skip = true;
			}

			if ( skip )
			{
				pPixel_b+= 4;
				pPixel_m+= 4;
				pPixel_bg+= 4;

				skip = false;
				continue;
			}

			r = pPixel_b[0];
			g = pPixel_b[1];
			b = pPixel_b[2];

			//AlphaMap Colorizing
			//Saturation Effect
			if( _saturation != 255 )
			{
				BYTE gray = (BYTE)(r * 0.3086 + g * 0.6094 + b * 0.0820);
				r = (BYTE)((r * _saturation + gray * (255 - _saturation) + 255) >> 8);
				g = (BYTE)((g * _saturation + gray * (255 - _saturation) + 255) >> 8);
				b = (BYTE)((b * _saturation + gray * (255 - _saturation) + 255) >> 8);
			}
			// Hue effect
			if( _hueIntensity != 0 )
			{
				if ( IsOS( OS_2K ) )
				{
					r = (BYTE)((r * (255 - _hueIntensity) + GetBValue(_hueColor) * _hueIntensity + 255) >> 8);
					g = (BYTE)((g * (255 - _hueIntensity) + GetGValue(_hueColor) * _hueIntensity + 255) >> 8);
					b = (BYTE)((b * (255 - _hueIntensity) + GetRValue(_hueColor) * _hueIntensity + 255) >> 8);
				}
				else if ( IsOS( OS_XP ) )
				{
					if ( pPixel_b[3] != 0 && pPixel_b[3] != 255 )
					{
						r = (BYTE)((r * (255 - _hueIntensity) + (pPixel_b[3]*GetBValue(_hueColor)/255) * _hueIntensity + 255) >> 8);
						g = (BYTE)((g * (255 - _hueIntensity) + (pPixel_b[3]*GetGValue(_hueColor)/255) * _hueIntensity + 255) >> 8);
						b = (BYTE)((b * (255 - _hueIntensity) + (pPixel_b[3]*GetRValue(_hueColor)/255) * _hueIntensity + 255) >> 8);
					}
					else
					{
						r = (BYTE)((r * (255 - _hueIntensity) + GetBValue(_hueColor) * _hueIntensity + 255) >> 8);
						g = (BYTE)((g * (255 - _hueIntensity) + GetGValue(_hueColor) * _hueIntensity + 255) >> 8);
						b = (BYTE)((b * (255 - _hueIntensity) + GetRValue(_hueColor) * _hueIntensity + 255) >> 8);
					}
				}
			}

			pPixel_b[0] = r;
			pPixel_b[1] = g;
			pPixel_b[2] = b;

			//Fix for Icons without AlphaChannel (all is 0!), with AlphaChannel 0 is Mask
			if ( pPixel_b[3] == 0 || pPixel_bg[3] == 255 )
				pPixel_b[3] = 255;

			pPixel_b+= 4;
			pPixel_m+= 4;
			pPixel_bg+= 4;
		}

		//BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, 255, AC_SRC_ALPHA };

		AlphaBlend(hDC, x+iconX, y+iconY, _iconSize, _iconSize, bufferDC, 0, 0, _iconSize, _iconSize, blendFunction);

		bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldbufferBitmap);
		DeleteObject(bufferBitmap);
		DeleteDC(bufferDC);

		maskBitmap = (HBITMAP) SelectObject(maskDC, oldmaskBitmap);
		DeleteObject(maskBitmap);
		DeleteDC(maskDC);

		BGBitmap = (HBITMAP) SelectObject(BGDC, oldBGBitmap);
		DeleteObject(BGBitmap);
		DeleteDC(BGDC);
	}
	else
	{
		DrawIconEx(hDC, x+iconX, y+iconY, hIcon, _iconSize, _iconSize, 0, NULL, DI_NORMAL);
	}
}

