/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2005 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "ButtonTexture.h"

NameValuePair modeConstants[] =
	{
		{ "stretch", MULTIBLT_STRETCH },
		{ "tile", MULTIBLT_TILE },
		{ "tile-horizontal", MULTIBLT_TILEHORIZONTAL },
		{ "tile-vertical", MULTIBLT_TILEVERTICAL },
		{ "tilehorizontal", MULTIBLT_TILEHORIZONTAL },
		{ "tilevertical", MULTIBLT_TILEVERTICAL },
		{ 0, 0 }
	};

NameValuePair borderValues[] = 
	{
		{ ".none", 0 },
		{ "none", 0 },
		{ "raised", 1 },
		{ "sunken", 2 },
		{ "etched", 3 },
		{ "bump", 4 },
		{ 0, 0 }
	};

#define COLOR_GRADIENTACTIVECAPTION     27
#define COLOR_GRADIENTINACTIVECAPTION   28

ButtonTexture::ButtonTexture(const string &name)
{
	transparencyNeeded = false;

	useAlphaMap = false;

	NormalGradient.clear();
	ActiveGradient.clear();
	MinimizedGradient.clear();
	FlashingGradient.clear();

	HoverGradient.clear();
	GroupGradient.clear();

	memoryNormalDC = 0;
	memoryNormalBitmap = 0;

	memoryActiveDC = 0;
	memoryActiveBitmap = 0;

	memoryMinimizedDC = 0;
	memoryMinimizedBitmap = 0;

	memoryFlashingDC = 0;
	memoryFlashingBitmap = 0; 

	memoryHoverDC = 0;
	memoryHoverBitmap = 0; 

	memoryGroupDC = 0;
	memoryGroupBitmap = 0; 

	normalContainsAlpha = false;
	activeContainsAlpha = false;
	minimizedContainsAlpha = false;
	flashingContainsAlpha = false;

	hoverContainsAlpha = false;
	groupContainsAlpha = false;

	defaultbutton.button_saturation = GetxRCInt(name , "ButtonSaturationIntensity", 255, 0, 255);
	defaultbutton.button_hueIntensity = GetxRCInt(name , "ButtonHueIntensity", 0, 0, 255);
	defaultbutton.button_hueColor = GetxRCColor(name, "ButtonHueColor", RGB(255, 255, 255));

	defaultbutton.borderMethod = GetxRCNamedValue(name, "ButtonBorderMethod", borderValues, 0);

	defaultbutton.leftEdge = GetxRCInt(name, "ButtonImageLeftEdge", 0, 0);
	defaultbutton.topEdge = GetxRCInt(name, "ButtonImageTopEdge", 0, 0);
	defaultbutton.rightEdge = GetxRCInt(name, "ButtonImageRightEdge", 0, 0);
	defaultbutton.bottomEdge = GetxRCInt(name, "ButtonImageBottomEdge", 0, 0);

	defaultbutton.mode = GetxRCNamedValue(name, "ButtonImageMode", modeConstants, MULTIBLT_STRETCH);

	defaultbutton.trueTransparent = GetxRCBool(name, "ButtonTrueTransparent");

	defaultbutton.alphaTransparency = GetxRCInt(name, "ButtonAlphaTransparency", 255, 0, 255);

	_defaultsolidcolors = GetxRCLine(name, "ButtonSolidColors", "");
	_defaultbevelsize = GetxRCLine(name, "ButtonSolidBevelSize", "0");
	_defaultgradientcolor = GetxRCLine(name, "ButtonSolidGradientColors", "");
	_defaultverticalgradient = GetxRCBool(name, "ButtonSolidGradientVertical");
	_defaultrepeatedgradient = GetxRCInt(name, "ButtonSolidGradientRepeated", 0);
}

ButtonTexture::~ButtonTexture()
{
	NormalGradient.clear();
	ActiveGradient.clear();
	MinimizedGradient.clear();
	FlashingGradient.clear();

	HoverGradient.clear();
	GroupGradient.clear();

	if(memoryNormalBitmap)
	{
		memoryNormalBitmap = (HBITMAP) SelectObject(memoryNormalDC, memoryNormalBitmap);
		DeleteObject(memoryNormalBitmap);
	}
	if(memoryNormalDC)
		DeleteDC(memoryNormalDC);

	if(memoryActiveBitmap)
	{
		memoryActiveBitmap = (HBITMAP) SelectObject(memoryActiveDC, memoryActiveBitmap);
		DeleteObject(memoryActiveBitmap);
	}
	if(memoryActiveDC)
		DeleteDC(memoryActiveDC);

	if(memoryMinimizedBitmap)
	{
		memoryMinimizedBitmap = (HBITMAP) SelectObject(memoryMinimizedDC, memoryMinimizedBitmap);
		DeleteObject(memoryMinimizedBitmap);
	}
	if(memoryMinimizedDC)
		DeleteDC(memoryMinimizedDC);

	if(memoryFlashingBitmap)
	{
		memoryFlashingBitmap = (HBITMAP) SelectObject(memoryFlashingDC, memoryFlashingBitmap);
		DeleteObject(memoryFlashingBitmap);
	}
	if(memoryFlashingDC)
		DeleteDC(memoryFlashingDC);

	if(memoryHoverBitmap)
	{
		memoryHoverBitmap = (HBITMAP) SelectObject(memoryHoverDC, memoryHoverBitmap);
		DeleteObject(memoryHoverBitmap);
	}
	if(memoryHoverDC)
		DeleteDC(memoryHoverDC);

	if(memoryGroupBitmap)
	{
		memoryGroupBitmap = (HBITMAP) SelectObject(memoryGroupDC, memoryGroupBitmap);
		DeleteObject(memoryGroupBitmap);
	}
	if(memoryGroupDC)
		DeleteDC(memoryGroupDC);
}

void ButtonTexture::parseCFG(string cfg, COLORREF &bgcolor, COLORREF &lightcolor, COLORREF &darkcolor, int imageType)
{
	bgcolor = RGB(255,255,255);
	lightcolor = GetSysColor(COLOR_3DLIGHT);
	darkcolor = GetSysColor(COLOR_3DSHADOW);
		
	if ( cfg.empty() )
		return;

	int color;
	char main[32], light[32], dark[32];
	char *buffers[] = {main, light, dark};

	int numTokens = LCTokenize(cfg.c_str(), buffers, 3, NULL);

	if(numTokens == 1)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		if (imageType == 0)
		{
			lightcolor = GetSysColor(COLOR_3DLIGHT);
			darkcolor = GetSysColor(COLOR_3DSHADOW);
		}
		else
		{
			lightcolor = normallightBevelColor;
			darkcolor = normaldarkBevelColor;
		}
	}
	else if (numTokens == 2)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(light, NULL, 16);
		lightcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		darkcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
	}
	else if (numTokens == 3)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(light, NULL, 16);
		lightcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(dark, NULL, 16);
		darkcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
	}
}

void ButtonTexture::parseGradientCFG(string cfg, int imageType)
{
	if ( cfg.empty() )
		return;

	int color;
	COLORREF insertColor;
	char token[4096] = {0};
	LPCSTR nextToken = cfg.c_str();
	
	while (GetToken(nextToken, token, &nextToken, false))
	{
		color = strtol(token, NULL, 16);
		insertColor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		if (imageType == 0)
		{
			NormalGradient.insert(NormalGradient.end(), insertColor );
		}
		else if (imageType == 1)
		{
			ActiveGradient.insert(ActiveGradient.end(), insertColor );
		}
		else if (imageType == 2)
		{
			MinimizedGradient.insert(MinimizedGradient.end(), insertColor );
		}
		else if (imageType == 3)
		{
			FlashingGradient.insert(FlashingGradient.end(), insertColor );
		}
		else if (imageType == 4)
		{
			HoverGradient.insert(HoverGradient.end(), insertColor );
		}
		else if (imageType == 5)
		{
			GroupGradient.insert(GroupGradient.end(), insertColor );
		}
	}
}

void ButtonTexture::parseBevelCFG(string cfg, int &leftBevel, int &rightBevel, int &topBevel, int &bottomBevel)
{
	char left[32], right[32], top[32], bottom[32];
	char *buffers[] = {left, right, top, bottom };

	leftBevel = 0;
	rightBevel = 0;
	topBevel = 0;
	bottomBevel = 0;

	int numTokens = LCTokenize(cfg.c_str(), buffers, 4, NULL);
		
	if(numTokens == 1)
	{
		leftBevel = atoi(left);
		rightBevel = leftBevel;
		topBevel = leftBevel;
		bottomBevel = leftBevel;
	}
	else if (numTokens == 2)
	{
		leftBevel = atoi(left);
		rightBevel = leftBevel;
		topBevel = atoi(right);
		bottomBevel = topBevel;
	}
	else if (numTokens == 4)
	{
		leftBevel = atoi(left);
		rightBevel = atoi(right);
		topBevel = atoi(top);
		bottomBevel = atoi(bottom);
	}
}

void ButtonTexture::subconfigure(string type, BUTTONSETTINGS &workSettings, BUTTONSETTINGS &defaultSettings)
{
	workSettings.button_saturation = GetxRCInt(type , "ButtonSaturationIntensity", defaultSettings.button_saturation, 0, 255);
	workSettings.button_hueIntensity = GetxRCInt(type , "ButtonHueIntensity", defaultSettings.button_hueIntensity, 0, 255);
	workSettings.button_hueColor = GetxRCColor(type, "ButtonHueColor", defaultSettings.button_hueColor);

	workSettings.borderMethod = GetxRCNamedValue(type, "ButtonBorderMethod", borderValues, defaultSettings.borderMethod);

	workSettings.leftEdge = GetxRCInt(type, "ButtonImageLeftEdge", defaultSettings.leftEdge, 0);
	workSettings.topEdge = GetxRCInt(type, "ButtonImageTopEdge", defaultSettings.topEdge, 0);
	workSettings.rightEdge = GetxRCInt(type, "ButtonImageRightEdge", defaultSettings.rightEdge, 0);
	workSettings.bottomEdge = GetxRCInt(type, "ButtonImageBottomEdge", defaultSettings.bottomEdge, 0);

	workSettings.mode = GetxRCNamedValue(type, "ButtonImageMode", modeConstants, defaultSettings.mode);

	workSettings.trueTransparent = GetxRCBool(type, "ButtonTrueTransparent", defaultSettings.trueTransparent);

	workSettings.alphaTransparency = GetxRCInt(type, "ButtonAlphaTransparency", defaultSettings.alphaTransparency, 0, 255);
}

void ButtonTexture::configure(const string &name, bool alphaMap)
{
	useAlphaMap = alphaMap;

	string imageFile = "";

	usenormalSolidColors = GetxRCBool(name, "UseNormalButtonSolidColors", GetxRCBool(name, "UseButtonSolidColors"));

	//To provide a SolidColor Default, if Normal ISN'T Set!
	_normalsolidcolors = _defaultsolidcolors;
	_normalgradientcolor = _defaultgradientcolor;
	_normalbevelsize = _defaultbevelsize;
	
	normalverticalGradient = _defaultverticalgradient;
	normalrepeatedGradient = _defaultrepeatedgradient;

	subconfigure((name + "Normal"), normal, defaultbutton);

	subconfigure((name + "Active"), active, normal);

	subconfigure((name + "Minimized"), minimized, normal);

	subconfigure((name + "Flashing"), flashing, normal);

	subconfigure((name + "Hover"), hover, normal);

	subconfigure((name + "Grouped"), group, normal);

	if (usenormalSolidColors)
	{
		_normalsolidcolors = GetxRCLine(name, "NormalButtonSolidColors", _defaultsolidcolors);
		if ( !_normalsolidcolors.empty() )
		{
			parseCFG(_normalsolidcolors, normalbackgroundColor, normallightBevelColor, normaldarkBevelColor, 0);
		}
		else
		{
			normalbackgroundColor = GetSysColor(COLOR_INACTIVECAPTION);
			normallightBevelColor = GetSysColor(COLOR_3DLIGHT);
			normaldarkBevelColor = GetSysColor(COLOR_3DSHADOW);
		}
		//BG Insert in Gradient List
		NormalGradient.insert(NormalGradient.end(), normalbackgroundColor );

		if (normalbackgroundColor == RGB(255,0,255) )
			transparencyNeeded = true; 

		_normalbevelsize = GetxRCLine(name, "NormalButtonSolidBevelSize", _defaultbevelsize);
		parseBevelCFG(_normalbevelsize, normalbevelSizeLeft, normalbevelSizeRight, normalbevelSizeTop, normalbevelSizeBottom);

		_normalgradientcolor = GetxRCLine(name, "NormalButtonSolidGradientColors", _defaultgradientcolor);
		if ( _normalsolidcolors.empty() && _normalgradientcolor.empty() )
			NormalGradient.insert(NormalGradient.end(), GetSysColor(COLOR_GRADIENTINACTIVECAPTION) );
		else
			parseGradientCFG(_normalgradientcolor, 0);
		
		normalverticalGradient = GetxRCBool(name, "NormalButtonSolidGradientVertical", _defaultverticalgradient);
		normalrepeatedGradient = GetxRCInt(name, "NormalButtonSolidGradientRepeated", _defaultrepeatedgradient, 0);
	}
	else
	{
		imageFile = GetxRCString(name, "ButtonImage", "");
		if (imageFile.empty())
		{
			imageFile = GetxRCString(name, "NormalButtonImage", "");
		}
		else if (!imageFile.empty())
		{
			string temp = GetxRCString(name, "NormalButtonImage", "");
			if ( !temp.empty() )
				imageFile = temp;
		}

		memoryNormalBitmap = LoadLSImage(imageFile.c_str(), 0);

		if (memoryNormalBitmap)
		{
			if (useAlphaMap)
			{
				BITMAP bm;
				GetObject(memoryNormalBitmap, sizeof(BITMAP), &bm);
				
				if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
				{
					PrepareAlphaChannel(memoryNormalBitmap, normal.button_saturation, normal.button_hueIntensity, normal.button_hueColor);
					normalContainsAlpha = true;
				}		
			}

			GetLSBitmapSize(memoryNormalBitmap, &imageNormalWidth, &imageNormalHeight);

			memoryNormalDC = CreateCompatibleDC(NULL);
			SelectObject(memoryNormalDC, memoryNormalBitmap);
		}
		else
		{
			transparencyNeeded = true;
		}
	}

	useactiveSolidColors = GetxRCBool(name, "UseActiveButtonSolidColors", usenormalSolidColors);

	if (useactiveSolidColors)
	{
		_activesolidcolors = GetxRCLine(name, "ActiveButtonSolidColors", _normalsolidcolors);
		if ( !_activesolidcolors.empty() )
		{
			parseCFG(_activesolidcolors, activebackgroundColor, activelightBevelColor, activedarkBevelColor, 1);
		}
		else
		{
			activebackgroundColor = GetSysColor(COLOR_ACTIVECAPTION);
			activelightBevelColor = GetSysColor(COLOR_3DLIGHT);
			activedarkBevelColor = GetSysColor(COLOR_3DSHADOW);
		}
		//BG Insert in Gradient List
		ActiveGradient.insert(ActiveGradient.end(), activebackgroundColor );

		if (activebackgroundColor == RGB(255,0,255) )
			transparencyNeeded = true; 

		imageFile = GetxRCLine(name, "ActiveButtonSolidBevelSize", _normalbevelsize);
		parseBevelCFG(imageFile, activebevelSizeLeft, activebevelSizeRight, activebevelSizeTop, activebevelSizeBottom);

		_activegradientcolor = GetxRCLine(name, "ActiveButtonSolidGradientColors", _normalgradientcolor);
		if ( _activesolidcolors.empty() && _activegradientcolor.empty() )
			ActiveGradient.insert(ActiveGradient.end(), GetSysColor(COLOR_GRADIENTACTIVECAPTION) );
		else
			parseGradientCFG(_activegradientcolor, 1);
		
		activeverticalGradient = GetxRCBool(name, "ActiveButtonSolidGradientVertical", normalverticalGradient);
		activerepeatedGradient = GetxRCInt(name, "ActiveButtonSolidGradientRepeated", normalrepeatedGradient, 0);
	}
	else
	{
		imageFile = GetxRCString(name, "ActiveButtonImage", "");
		if (!imageFile.empty())
		{
			memoryActiveBitmap = LoadLSImage(imageFile.c_str(), 0);

			if (memoryActiveBitmap)
			{
				if (useAlphaMap)
				{
					BITMAP bm;
					GetObject(memoryActiveBitmap, sizeof(BITMAP), &bm);
					
					if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
					{
						PrepareAlphaChannel(memoryActiveBitmap, active.button_saturation, active.button_hueIntensity, active.button_hueColor);
						activeContainsAlpha = true;
					}	
				}

				GetLSBitmapSize(memoryActiveBitmap, &imageActiveWidth, &imageActiveHeight);

				memoryActiveDC = CreateCompatibleDC(NULL);
				SelectObject(memoryActiveDC, memoryActiveBitmap);
			}
		}
	}

	useminimizedSolidColors = GetxRCBool(name, "UseMinimizedButtonSolidColors", usenormalSolidColors);

	if (useminimizedSolidColors)
	{
		imageFile = GetxRCLine(name, "MinimizedButtonSolidColors", _normalsolidcolors);
		if ( !imageFile.empty() )
		{
			parseCFG(imageFile, minimizedbackgroundColor, minimizedlightBevelColor, minimizeddarkBevelColor, 2);
		}
		else
		{
			minimizedbackgroundColor = normalbackgroundColor;
			minimizedlightBevelColor = normallightBevelColor;
			minimizeddarkBevelColor = normaldarkBevelColor;
		}
		//BG Insert in Gradient List
		MinimizedGradient.insert(MinimizedGradient.end(), minimizedbackgroundColor );

		if (minimizedbackgroundColor == RGB(255,0,255) )
			transparencyNeeded = true; 

		imageFile = GetxRCLine(name, "MinimizedButtonSolidBevelSize", _normalbevelsize);
		parseBevelCFG(imageFile, minimizedbevelSizeLeft, minimizedbevelSizeRight, minimizedbevelSizeTop, minimizedbevelSizeBottom);

		parseGradientCFG(GetxRCLine(name, "MinimizedButtonSolidGradientColors", _normalgradientcolor), 2);

		minimizedverticalGradient = GetxRCBool(name, "MinimizedButtonSolidGradientVertical", normalverticalGradient);
		minimizedrepeatedGradient = GetxRCInt(name, "MinimizedButtonSolidGradientRepeated", normalrepeatedGradient, 0);
	}
	else
	{
		imageFile = GetxRCString(name, "MinimizedButtonImage", "");
		if (!imageFile.empty())
		{
			memoryMinimizedBitmap = LoadLSImage(imageFile.c_str(), 0);

			if (memoryMinimizedBitmap)
			{
				if (useAlphaMap)
				{
					BITMAP bm;
					GetObject(memoryMinimizedBitmap, sizeof(BITMAP), &bm);
					
					if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
					{
						PrepareAlphaChannel(memoryMinimizedBitmap, minimized.button_saturation, minimized.button_hueIntensity, minimized.button_hueColor);
						minimizedContainsAlpha = true;
					}		
				}

				GetLSBitmapSize(memoryMinimizedBitmap, &imageMinimizedWidth, &imageMinimizedHeight);

				memoryMinimizedDC = CreateCompatibleDC(NULL);
				SelectObject(memoryMinimizedDC, memoryMinimizedBitmap);
			}
		}
	}

	useflashingSolidColors = GetxRCBool(name, "UseFlashingButtonSolidColors", usenormalSolidColors);

	if (useflashingSolidColors)
	{
		imageFile = GetxRCLine(name, "FlashingButtonSolidColors", _normalsolidcolors);
		if ( !imageFile.empty() )
		{
			parseCFG(imageFile, flashingbackgroundColor, flashinglightBevelColor, flashingdarkBevelColor, 3);
		}
		else
		{
			flashingbackgroundColor = activebackgroundColor;
			flashinglightBevelColor = activelightBevelColor;
			flashingdarkBevelColor = activedarkBevelColor;
		}
		//BG Insert in Gradient List
		FlashingGradient.insert(FlashingGradient.end(), flashingbackgroundColor );

		if (flashingbackgroundColor == RGB(255,0,255) )
			transparencyNeeded = true; 

		imageFile = GetxRCLine(name, "FlashingButtonSolidBevelSize", _normalbevelsize);
		parseBevelCFG(imageFile, flashingbevelSizeLeft, flashingbevelSizeRight, flashingbevelSizeTop, flashingbevelSizeBottom);

		parseGradientCFG(GetxRCLine(name, "FlashingButtonSolidGradientColors", _normalgradientcolor), 3);

		flashingverticalGradient = GetxRCBool(name, "FlashingButtonSolidGradientVertical", normalverticalGradient);
		flashingrepeatedGradient = GetxRCInt(name, "FlashingButtonSolidGradientRepeated", normalrepeatedGradient, 0);
	}
	else
	{
		imageFile = GetxRCString(name, "FlashingButtonImage", "");
		if (!imageFile.empty())
		{
			memoryFlashingBitmap = LoadLSImage(imageFile.c_str(), 0);

			if (memoryFlashingBitmap)
			{
				if (useAlphaMap)
				{
					BITMAP bm;
					GetObject(memoryFlashingBitmap, sizeof(BITMAP), &bm);
					
					if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
					{
						PrepareAlphaChannel(memoryFlashingBitmap, flashing.button_saturation, flashing.button_hueIntensity, flashing.button_hueColor);
						flashingContainsAlpha = true;
					}		
				}

				GetLSBitmapSize(memoryFlashingBitmap, &imageFlashingWidth, &imageFlashingHeight);

				memoryFlashingDC = CreateCompatibleDC(NULL);
				SelectObject(memoryFlashingDC, memoryFlashingBitmap);
			}
		}
	}

	usehoverSolidColors = GetxRCBool(name, "UseHoverButtonSolidColors");

	if (usehoverSolidColors)
	{
		imageFile = GetxRCLine(name, "HoverButtonSolidColors", _defaultsolidcolors);
		parseCFG(imageFile, hoverbackgroundColor, hoverlightBevelColor, hoverdarkBevelColor, 4);
		//BG Insert in Gradient List
		HoverGradient.insert(HoverGradient.end(), hoverbackgroundColor );

		if (hoverbackgroundColor == RGB(255,0,255) )
			transparencyNeeded = true; 

		imageFile = GetxRCLine(name, "HoverButtonSolidBevelSize", _defaultbevelsize);
		parseBevelCFG(imageFile, hoverbevelSizeLeft, hoverbevelSizeRight, hoverbevelSizeTop, hoverbevelSizeBottom);

		parseGradientCFG(GetxRCLine(name, "HoverButtonSolidGradientColors", _defaultgradientcolor), 4);

		hoververticalGradient = GetxRCBool(name, "HoverButtonSolidGradientVertical", _defaultverticalgradient);
		hoverrepeatedGradient = GetxRCInt(name, "HoverButtonSolidGradientRepeated", _defaultrepeatedgradient, 0);
	}
	else
	{
		imageFile = GetxRCString(name, "HoverButtonImage", "");
		if (!imageFile.empty())
		{
			memoryHoverBitmap = LoadLSImage(imageFile.c_str(), 0);

			if (memoryHoverBitmap)
			{
				if (useAlphaMap)
				{
					BITMAP bm;
					GetObject(memoryHoverBitmap, sizeof(BITMAP), &bm);
					
					if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
					{
						PrepareAlphaChannel(memoryHoverBitmap, hover.button_saturation, hover.button_hueIntensity, hover.button_hueColor);
						hoverContainsAlpha = true;
					}	
				}

				GetLSBitmapSize(memoryHoverBitmap, &imageHoverWidth, &imageHoverHeight);

				memoryHoverDC = CreateCompatibleDC(NULL);
				SelectObject(memoryHoverDC, memoryHoverBitmap);
			}
		}
	}

	usegroupSolidColors = GetxRCBool(name, "UseGroupedButtonSolidColors");

	if (usegroupSolidColors)
	{
		imageFile = GetxRCLine(name, "GroupedButtonSolidColors", _defaultsolidcolors);
		parseCFG(imageFile, groupbackgroundColor, grouplightBevelColor, groupdarkBevelColor, 5);
		//BG Insert in Gradient List
		GroupGradient.insert(GroupGradient.end(), groupbackgroundColor );

		if (groupbackgroundColor == RGB(255,0,255) )
			transparencyNeeded = true; 

		imageFile = GetxRCLine(name, "GroupedButtonSolidBevelSize", _defaultbevelsize);
		parseBevelCFG(imageFile, groupbevelSizeLeft, groupbevelSizeRight, groupbevelSizeTop, groupbevelSizeBottom);

		parseGradientCFG(GetxRCLine(name, "GroupedButtonSolidGradientColors", _defaultgradientcolor), 5);

		groupverticalGradient = GetxRCBool(name, "GroupedButtonSolidGradientVertical", _defaultverticalgradient);
		grouprepeatedGradient = GetxRCInt(name, "GroupedButtonSolidGradientRepeated", _defaultrepeatedgradient, 0);
	}
	else
	{
		imageFile = GetxRCString(name, "GroupedButtonImage", "");
		if (!imageFile.empty())
		{
			memoryGroupBitmap = LoadLSImage(imageFile.c_str(), 0);

			if (memoryGroupBitmap)
			{
				if (useAlphaMap)
				{
					BITMAP bm;
					GetObject(memoryGroupBitmap, sizeof(BITMAP), &bm);
					
					if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
					{
						PrepareAlphaChannel(memoryGroupBitmap, group.button_saturation, group.button_hueIntensity, group.button_hueColor);
						groupContainsAlpha = true;
					}	
				}

				GetLSBitmapSize(memoryGroupBitmap, &imageGroupWidth, &imageGroupHeight);

				memoryGroupDC = CreateCompatibleDC(NULL);
				SelectObject(memoryGroupDC, memoryGroupBitmap);
			}
		}
	}

	if ( !useAlphaMap )
	{
		if (memoryNormalDC && !usenormalSolidColors)
			paintEffects(memoryNormalDC, imageNormalWidth, imageNormalHeight, normal.button_saturation, normal.button_hueIntensity, normal.button_hueColor);
		if (memoryActiveDC && !useactiveSolidColors)
			paintEffects(memoryActiveDC, imageActiveWidth, imageActiveHeight, active.button_saturation, active.button_hueIntensity, active.button_hueColor);
		if (memoryMinimizedDC && !useminimizedSolidColors)
			paintEffects(memoryMinimizedDC, imageMinimizedWidth, imageMinimizedHeight, minimized.button_saturation, minimized.button_hueIntensity, minimized.button_hueColor);
		if (memoryFlashingDC && !useflashingSolidColors)
			paintEffects(memoryFlashingDC, imageFlashingWidth, imageFlashingHeight, flashing.button_saturation, flashing.button_hueIntensity, flashing.button_hueColor);
		if (memoryHoverDC && !usehoverSolidColors)
			paintEffects(memoryHoverDC, imageHoverWidth, imageHoverHeight, hover.button_saturation, hover.button_hueIntensity, hover.button_hueColor);
		if (memoryGroupDC && !usegroupSolidColors)
			paintEffects(memoryGroupDC, imageGroupWidth, imageGroupHeight, group.button_saturation, group.button_hueIntensity, group.button_hueColor);
	}
}

bool ButtonTexture::useAlphaTransparency(int buttonType)
{
	if ( buttonType == 0 && normal.alphaTransparency != 255 )
		return true;
	else if ( buttonType == 1 && active.alphaTransparency != 255 )
		return true;
	else if ( buttonType == 2 && minimized.alphaTransparency != 255 )
		return true;
	else if ( buttonType == 3 && flashing.alphaTransparency != 255 )
		return true;
	else if (buttonType == 4 && hover.alphaTransparency != 255 )
		return true;
	else if (buttonType == 5 && group.alphaTransparency != 255 )
		return true;

	return false;
}

void ButtonTexture::apply(HDC hDC, int x, int y, int width, int height, int buttonType)
{
	RECT r;
	r.left = 0;
	r.top = 0;
	r.right = width;
	r.bottom = height;

	int usedBorderMethod = 0;
	bool useTrueTransparent = false;

	int bevelWidth = 0;
	int bevelHeight = 0;

	SetStretchBltMode(hDC, STRETCH_DELETESCANS);

	if (buttonType == 0 && usenormalSolidColors)
	{
		HDC bufferDC = CreateCompatibleDC(hDC);
		HBITMAP bufferBitmap = CreateCompatibleBitmap(hDC, width, height);
		HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

		usedBorderMethod = normal.borderMethod;
		useTrueTransparent = normal.trueTransparent;

		bevelWidth = normalbevelSizeLeft+normalbevelSizeRight;
		bevelHeight = normalbevelSizeTop+normalbevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, normaldarkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-normalbevelSizeRight;
			r.bottom = r.bottom-normalbevelSizeBottom;
			::SetBkColor(bufferDC, normallightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+normalbevelSizeLeft;
			r.top = r.top+normalbevelSizeTop;

			if ( NormalGradient.size() > 1 )
			{
				solidGradientPaint(bufferDC, NormalGradient, r, 0);
			}
			else
			{
				::SetBkColor(bufferDC, normalbackgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( NormalGradient.size() > 1 )
			{
				solidGradientPaint(bufferDC, NormalGradient, r, 0);
			}
			else
			{
				::SetBkColor(bufferDC, normalbackgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}

		if ( useTrueTransparent )
		{
			BitBlt(hDC, x, y, width, height, bufferDC, 0, 0, SRCCOPY);
		}
		else
		{
			TransparentBltLS(hDC, x, y, width, height,
									bufferDC, 0, 0,
									RGB(255, 0, 255));
		}

		bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
		DeleteObject(bufferBitmap);
		DeleteDC(bufferDC);
	}
	else if ( buttonType == 1 && useactiveSolidColors )
	{
		HDC bufferDC = CreateCompatibleDC(hDC);
		HBITMAP bufferBitmap = CreateCompatibleBitmap(hDC, width, height);
		HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

		usedBorderMethod = active.borderMethod;
		useTrueTransparent = active.trueTransparent;

		bevelWidth = activebevelSizeLeft+activebevelSizeRight;
		bevelHeight = activebevelSizeTop+activebevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, activedarkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-activebevelSizeRight;
			r.bottom = r.bottom-activebevelSizeBottom;
			::SetBkColor(bufferDC, activelightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+activebevelSizeLeft;
			r.top = r.top+activebevelSizeTop;

			if ( ActiveGradient.size() > 1 )
			{
				solidGradientPaint(bufferDC, ActiveGradient, r, 1);
			}
			else
			{
				::SetBkColor(bufferDC, activebackgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( ActiveGradient.size() > 1 )
			{
				solidGradientPaint(bufferDC, ActiveGradient, r, 1);
			}
			else
			{
				::SetBkColor(bufferDC, activebackgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}

		if ( useTrueTransparent )
		{
			BitBlt(hDC, x, y, width, height, bufferDC, 0, 0, SRCCOPY);
		}
		else
		{
			TransparentBltLS(hDC, x, y, width, height,
									bufferDC, 0, 0,
									RGB(255, 0, 255));
		}

		bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
		DeleteObject(bufferBitmap);
		DeleteDC(bufferDC);
	}
	else if ( buttonType == 2 && useminimizedSolidColors )
	{
		HDC bufferDC = CreateCompatibleDC(hDC);
		HBITMAP bufferBitmap = CreateCompatibleBitmap(hDC, width, height);
		HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

		usedBorderMethod = minimized.borderMethod;
		useTrueTransparent = minimized.trueTransparent;

		bevelWidth = minimizedbevelSizeLeft+minimizedbevelSizeRight;
		bevelHeight = minimizedbevelSizeTop+minimizedbevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, minimizeddarkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-minimizedbevelSizeRight;
			r.bottom = r.bottom-minimizedbevelSizeBottom;
			::SetBkColor(bufferDC, minimizedlightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+minimizedbevelSizeLeft;
			r.top = r.top+minimizedbevelSizeTop;

			if ( MinimizedGradient.size() > 1 )
			{
				solidGradientPaint(bufferDC, MinimizedGradient, r, 2);
			}
			else
			{
				::SetBkColor(bufferDC, minimizedbackgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( MinimizedGradient.size() > 1 )
			{
				solidGradientPaint(bufferDC, MinimizedGradient, r, 2);
			}
			else
			{
				::SetBkColor(bufferDC, minimizedbackgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}

		if ( useTrueTransparent )
		{
			BitBlt(hDC, x, y, width, height, bufferDC, 0, 0, SRCCOPY);
		}
		else
		{
			TransparentBltLS(hDC, x, y, width, height,
									bufferDC, 0, 0,
									RGB(255, 0, 255));
		}

		bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
		DeleteObject(bufferBitmap);
		DeleteDC(bufferDC);
	}
	else if ( buttonType == 3 && useflashingSolidColors )
	{
		HDC bufferDC = CreateCompatibleDC(hDC);
		HBITMAP bufferBitmap = CreateCompatibleBitmap(hDC, width, height);
		HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

		usedBorderMethod = flashing.borderMethod;
		useTrueTransparent = flashing.trueTransparent;

		bevelWidth = flashingbevelSizeLeft+flashingbevelSizeRight;
		bevelHeight = flashingbevelSizeTop+flashingbevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, flashingdarkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-flashingbevelSizeRight;
			r.bottom = r.bottom-flashingbevelSizeBottom;
			::SetBkColor(bufferDC, flashinglightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+flashingbevelSizeLeft;
			r.top = r.top+flashingbevelSizeTop;

			if ( FlashingGradient.size() > 1 )
			{
				solidGradientPaint(bufferDC, FlashingGradient, r, 3);
			}
			else
			{
				::SetBkColor(bufferDC, flashingbackgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( FlashingGradient.size() > 1 )
			{
				solidGradientPaint(bufferDC, FlashingGradient, r, 3);
			}
			else
			{
				::SetBkColor(bufferDC, flashingbackgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}

		if ( useTrueTransparent )
		{
			BitBlt(hDC, x, y, width, height, bufferDC, 0, 0, SRCCOPY);
		}
		else
		{
			TransparentBltLS(hDC, x, y, width, height,
									bufferDC, 0, 0,
									RGB(255, 0, 255));
		}

		bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
		DeleteObject(bufferBitmap);
		DeleteDC(bufferDC);
	}
	else if ( buttonType == 4 && usehoverSolidColors )
	{
		HDC bufferDC = CreateCompatibleDC(hDC);
		HBITMAP bufferBitmap = CreateCompatibleBitmap(hDC, width, height);
		HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

		usedBorderMethod = hover.borderMethod;
		useTrueTransparent = hover.trueTransparent;

		bevelWidth = hoverbevelSizeLeft+hoverbevelSizeRight;
		bevelHeight = hoverbevelSizeTop+hoverbevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, hoverdarkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-hoverbevelSizeRight;
			r.bottom = r.bottom-hoverbevelSizeBottom;
			::SetBkColor(bufferDC, hoverlightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+hoverbevelSizeLeft;
			r.top = r.top+hoverbevelSizeTop;

			if ( HoverGradient.size() > 1 )
			{
				solidGradientPaint(bufferDC, HoverGradient, r, 4);
			}
			else
			{
				::SetBkColor(bufferDC, hoverbackgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( HoverGradient.size() > 1 )
			{
				solidGradientPaint(bufferDC, HoverGradient, r, 4);
			}
			else
			{
				::SetBkColor(bufferDC, hoverbackgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}

		if ( useTrueTransparent )
		{
			BitBlt(hDC, x, y, width, height, bufferDC, 0, 0, SRCCOPY);
		}
		else
		{
			TransparentBltLS(hDC, x, y, width, height,
									bufferDC, 0, 0,
									RGB(255, 0, 255));
		}

		bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
		DeleteObject(bufferBitmap);
		DeleteDC(bufferDC);
	}
	else if ( buttonType == 5 && usegroupSolidColors )
	{
		HDC bufferDC = CreateCompatibleDC(hDC);
		HBITMAP bufferBitmap = CreateCompatibleBitmap(hDC, width, height);
		HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

		usedBorderMethod = group.borderMethod;
		useTrueTransparent = group.trueTransparent;

		bevelWidth = groupbevelSizeLeft+groupbevelSizeRight;
		bevelHeight = groupbevelSizeTop+groupbevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, groupdarkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-groupbevelSizeRight;
			r.bottom = r.bottom-groupbevelSizeBottom;
			::SetBkColor(bufferDC, grouplightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+groupbevelSizeLeft;
			r.top = r.top+groupbevelSizeTop;

			if ( GroupGradient.size() > 1 )
			{
				solidGradientPaint(bufferDC, GroupGradient, r, 5);
			}
			else
			{
				::SetBkColor(bufferDC, groupbackgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( GroupGradient.size() > 1 )
			{
				solidGradientPaint(bufferDC, GroupGradient, r, 5);
			}
			else
			{
				::SetBkColor(bufferDC, groupbackgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}

		if ( useTrueTransparent )
		{
			BitBlt(hDC, x, y, width, height, bufferDC, 0, 0, SRCCOPY);
		}
		else
		{
			TransparentBltLS(hDC, x, y, width, height,
									bufferDC, 0, 0,
									RGB(255, 0, 255));
		}

		bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
		DeleteObject(bufferBitmap);
		DeleteDC(bufferDC);
	}
	else
	{
		if (buttonType == 4 && memoryHoverBitmap == 0)
			return;
		else if (buttonType == 5 && memoryGroupBitmap == 0)
			return;
	
		if ( buttonType == 1 && memoryActiveBitmap == 0 )
			buttonType = 0;
		else if ( buttonType == 2 && memoryMinimizedBitmap == 0 )
			buttonType = 0;
		else if ( buttonType == 3 && memoryFlashingBitmap == 0 )
			buttonType = 0;
		else if ( buttonType == 3 && memoryFlashingBitmap == 0 )
			buttonType = 0;

		if (buttonType == 0 && memoryNormalBitmap == 0)
			return;

		HDC bufferDC = CreateCompatibleDC(hDC);
		HBITMAP bufferBitmap = CreateCompatibleBitmap(hDC, width, height);
		HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

		bool useButtonAlphaMap = false;

		if (buttonType == 0)
		{
			usedBorderMethod = normal.borderMethod;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryNormalDC, 0, 0, imageNormalWidth, imageNormalHeight,
					normal.leftEdge, normal.topEdge, normal.rightEdge, normal.bottomEdge,
					normal.mode, SRCCOPY);

			useTrueTransparent = normal.trueTransparent;
			useButtonAlphaMap = normalContainsAlpha;
		}
		else if (buttonType == 1)
		{
			usedBorderMethod = active.borderMethod;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryActiveDC, 0, 0, imageActiveWidth, imageActiveHeight,
					active.leftEdge, active.topEdge, active.rightEdge, active.bottomEdge,
					active.mode, SRCCOPY);

			useTrueTransparent = active.trueTransparent;
			useButtonAlphaMap = activeContainsAlpha;
		}
		else if (buttonType == 2)
		{
			usedBorderMethod = minimized.borderMethod;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryMinimizedDC, 0, 0, imageMinimizedWidth, imageMinimizedHeight,
					minimized.leftEdge, minimized.topEdge, minimized.rightEdge, minimized.bottomEdge,
					minimized.mode, SRCCOPY);

			useTrueTransparent = minimized.trueTransparent;
			useButtonAlphaMap = minimizedContainsAlpha;
		}
		else if (buttonType == 3)
		{
			usedBorderMethod = flashing.borderMethod;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryFlashingDC, 0, 0, imageFlashingWidth, imageFlashingHeight,
					flashing.leftEdge, flashing.topEdge, flashing.rightEdge, flashing.bottomEdge,
					flashing.mode, SRCCOPY);

			useTrueTransparent = flashing.trueTransparent;
			useButtonAlphaMap = flashingContainsAlpha;
		}
		else if (buttonType == 4)
		{
			usedBorderMethod = hover.borderMethod;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryHoverDC, 0, 0, imageHoverWidth, imageHoverHeight,
					hover.leftEdge, hover.topEdge, hover.rightEdge, hover.bottomEdge,
					hover.mode, SRCCOPY);

			useTrueTransparent = hover.trueTransparent;
			useButtonAlphaMap = hoverContainsAlpha;
		}
		else if (buttonType == 5)
		{
			usedBorderMethod = group.borderMethod;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryGroupDC, 0, 0, imageGroupWidth, imageGroupHeight,
					group.leftEdge, group.topEdge, group.rightEdge, group.bottomEdge,
					group.mode, SRCCOPY);

			useTrueTransparent = group.trueTransparent;
			useButtonAlphaMap = groupContainsAlpha;
		}

		if (useAlphaMap && useButtonAlphaMap)
		{
			BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, 255, (useTrueTransparent ? 0 : AC_SRC_ALPHA) };

			AlphaBlend(hDC, x, y, width, height, bufferDC, 0, 0, width, height, blendFunction);
		}
		else
		{
			if ( useTrueTransparent )
			{
				BitBlt(hDC, x, y, width, height, bufferDC, 0, 0, SRCCOPY);
			}
			else
			{
				TransparentBltLS(hDC, x, y, width, height,
									bufferDC, 0, 0,
									RGB(255, 0, 255));
			}
		}

		bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
		DeleteObject(bufferBitmap);
		DeleteDC(bufferDC);
	}

	RECT completeFillRect = {x, y, x+width, y+height};
	if (usedBorderMethod == 1)
		DrawEdge(hDC, &completeFillRect, EDGE_RAISED, BF_RECT | BF_SOFT);
	else if (usedBorderMethod == 2)
		DrawEdge(hDC, &completeFillRect, EDGE_SUNKEN, BF_RECT | BF_SOFT);
	else if (usedBorderMethod == 3)
		DrawEdge(hDC, &completeFillRect, EDGE_ETCHED, BF_RECT | BF_SOFT);
	else if (usedBorderMethod == 4)
		DrawEdge(hDC, &completeFillRect, EDGE_BUMP, BF_RECT | BF_SOFT);
}

void ButtonTexture::applyOverlay(HDC hDC, int x, int y, int width, int height, int buttonType)
{
	RECT r;
	r.left = 0;
	r.top = 0;
	r.right = width;
	r.bottom = height;

	int usedAlphaTransparency = 255;
	int usedBorderMethod = 0;
	bool useTrueTransparent = false;

	int bevelWidth = 0;
	int bevelHeight = 0;

	HDC bufferDC = CreateCompatibleDC(NULL);

	BITMAPINFO alpha;
	ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
	alpha.bmiHeader.biWidth=width;      // Set size you need
	alpha.bmiHeader.biHeight=height;    // Set size you need
	alpha.bmiHeader.biPlanes=1;
	alpha.bmiHeader.biBitCount=32;      // Can be 8, 16, 32 bpp or other number
	alpha.bmiHeader.biSizeImage=0;
	alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
	alpha.bmiHeader.biClrUsed= 0;
	alpha.bmiHeader.biClrImportant= 0;
	VOID *pvBits;

	HBITMAP bufferBitmap = CreateDIBSection( bufferDC,
												&alpha,
												DIB_RGB_COLORS,
												&pvBits,
												NULL,
												0 );

	HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

	// set source alpha to zero
    /*for (int iy = 0; iy < height; iy++)
        for (int ix = 0; ix < width; ix++)
            ((UINT32 *)pvBits)[ix + iy * width] = 0x00000000;*/

	SetStretchBltMode(bufferDC, STRETCH_DELETESCANS);

	if (buttonType == 0 && usenormalSolidColors)
	{
		usedAlphaTransparency = normal.alphaTransparency;
		usedBorderMethod = normal.borderMethod;
		useTrueTransparent = normal.trueTransparent;

		bevelWidth = normalbevelSizeLeft+normalbevelSizeRight;
		bevelHeight = normalbevelSizeTop+normalbevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, normaldarkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-normalbevelSizeRight;
			r.bottom = r.bottom-normalbevelSizeBottom;
			::SetBkColor(bufferDC, normallightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+normalbevelSizeLeft;
			r.top = r.top+normalbevelSizeTop;

			if ( NormalGradient.size() > 1 )
			{
				solidGradientPaint(bufferDC, NormalGradient, r, 0);
			}
			else
			{
				::SetBkColor(bufferDC, normalbackgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( NormalGradient.size() > 1 )
			{
				solidGradientPaint(bufferDC, NormalGradient, r, 0);
			}
			else
			{
				::SetBkColor(bufferDC, normalbackgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 1 && useactiveSolidColors )
	{
		usedAlphaTransparency = active.alphaTransparency;
		usedBorderMethod = active.borderMethod;
		useTrueTransparent = active.trueTransparent;

		bevelWidth = activebevelSizeLeft+activebevelSizeRight;
		bevelHeight = activebevelSizeTop+activebevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, activedarkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-activebevelSizeRight;
			r.bottom = r.bottom-activebevelSizeBottom;
			::SetBkColor(bufferDC, activelightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+activebevelSizeLeft;
			r.top = r.top+activebevelSizeTop;

			if ( ActiveGradient.size() > 1 )
			{
				solidGradientPaint(bufferDC, ActiveGradient, r, 1);
			}
			else
			{
				::SetBkColor(bufferDC, activebackgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( ActiveGradient.size() > 1 )
			{
				solidGradientPaint(bufferDC, ActiveGradient, r, 1);
			}
			else
			{
				::SetBkColor(bufferDC, activebackgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 2 && useminimizedSolidColors )
	{
		usedAlphaTransparency = minimized.alphaTransparency;
		usedBorderMethod = minimized.borderMethod;
		useTrueTransparent = minimized.trueTransparent;

		bevelWidth = minimizedbevelSizeLeft+minimizedbevelSizeRight;
		bevelHeight = minimizedbevelSizeTop+minimizedbevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, minimizeddarkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-minimizedbevelSizeRight;
			r.bottom = r.bottom-minimizedbevelSizeBottom;
			::SetBkColor(bufferDC, minimizedlightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+minimizedbevelSizeLeft;
			r.top = r.top+minimizedbevelSizeTop;

			if ( MinimizedGradient.size() > 1 )
			{
				solidGradientPaint(bufferDC, MinimizedGradient, r, 2);
			}
			else
			{
				::SetBkColor(bufferDC, minimizedbackgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( MinimizedGradient.size() > 1 )
			{
				solidGradientPaint(bufferDC, MinimizedGradient, r, 2);
			}
			else
			{
				::SetBkColor(bufferDC, minimizedbackgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 3 && useflashingSolidColors )
	{
		usedAlphaTransparency = flashing.alphaTransparency;
		usedBorderMethod = flashing.borderMethod;
		useTrueTransparent = flashing.trueTransparent;

		bevelWidth = flashingbevelSizeLeft+flashingbevelSizeRight;
		bevelHeight = flashingbevelSizeTop+flashingbevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, flashingdarkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-flashingbevelSizeRight;
			r.bottom = r.bottom-flashingbevelSizeBottom;
			::SetBkColor(bufferDC, flashinglightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+flashingbevelSizeLeft;
			r.top = r.top+flashingbevelSizeTop;

			if ( FlashingGradient.size() > 1 )
			{
				solidGradientPaint(bufferDC, FlashingGradient, r, 3);
			}
			else
			{
				::SetBkColor(bufferDC, flashingbackgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( FlashingGradient.size() > 1 )
			{
				solidGradientPaint(bufferDC, FlashingGradient, r, 3);
			}
			else
			{
				::SetBkColor(bufferDC, flashingbackgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 4 && usehoverSolidColors )
	{
		usedAlphaTransparency = hover.alphaTransparency;
		usedBorderMethod = hover.borderMethod;
		useTrueTransparent = hover.trueTransparent;

		bevelWidth = hoverbevelSizeLeft+hoverbevelSizeRight;
		bevelHeight = hoverbevelSizeTop+hoverbevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, hoverdarkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-hoverbevelSizeRight;
			r.bottom = r.bottom-hoverbevelSizeBottom;
			::SetBkColor(bufferDC, hoverlightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+hoverbevelSizeLeft;
			r.top = r.top+hoverbevelSizeTop;

			if ( HoverGradient.size() > 1 )
			{
				solidGradientPaint(bufferDC, HoverGradient, r, 4);
			}
			else
			{
				::SetBkColor(bufferDC, hoverbackgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( HoverGradient.size() > 1 )
			{
				solidGradientPaint(bufferDC, HoverGradient, r, 4);
			}
			else
			{
				::SetBkColor(bufferDC, hoverbackgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 5 && usegroupSolidColors )
	{
		usedAlphaTransparency = group.alphaTransparency;
		usedBorderMethod = group.borderMethod;
		useTrueTransparent = group.trueTransparent;

		bevelWidth = groupbevelSizeLeft+groupbevelSizeRight;
		bevelHeight = groupbevelSizeTop+groupbevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, groupdarkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-groupbevelSizeRight;
			r.bottom = r.bottom-groupbevelSizeBottom;
			::SetBkColor(bufferDC, grouplightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+groupbevelSizeLeft;
			r.top = r.top+groupbevelSizeTop;

			if ( GroupGradient.size() > 1 )
			{
				solidGradientPaint(bufferDC, GroupGradient, r, 5);
			}
			else
			{
				::SetBkColor(bufferDC, groupbackgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( GroupGradient.size() > 1 )
			{
				solidGradientPaint(bufferDC, GroupGradient, r, 5);
			}
			else
			{
				::SetBkColor(bufferDC, groupbackgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else
	{
		if (buttonType == 4 && memoryHoverBitmap == 0)
		{
			bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
			DeleteObject(bufferBitmap);
			DeleteDC(bufferDC);
			return;
		}
		else if (buttonType == 5 && memoryGroupBitmap == 0)
		{
			bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
			DeleteObject(bufferBitmap);
			DeleteDC(bufferDC);
			return;
		}
	
		if ( buttonType == 1 && memoryActiveBitmap == 0 )
		{
			bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
			DeleteObject(bufferBitmap);
			DeleteDC(bufferDC);
			return;
		}
		else if ( buttonType == 2 && memoryMinimizedBitmap == 0 )
		{
			bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
			DeleteObject(bufferBitmap);
			DeleteDC(bufferDC);
			return;
		}
		else if ( buttonType == 3 && memoryFlashingBitmap == 0 )
		{
			bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
			DeleteObject(bufferBitmap);
			DeleteDC(bufferDC);
			return;
		}
		else if ( buttonType == 3 && memoryFlashingBitmap == 0 )
		{
			bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
			DeleteObject(bufferBitmap);
			DeleteDC(bufferDC);
			return;
		}

		if (buttonType == 0 && memoryNormalBitmap == 0)
		{
			bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
			DeleteObject(bufferBitmap);
			DeleteDC(bufferDC);
			return;
		}

		if (buttonType == 0)
		{
			usedAlphaTransparency = normal.alphaTransparency;
			usedBorderMethod = normal.borderMethod;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryNormalDC, 0, 0, imageNormalWidth, imageNormalHeight,
					normal.leftEdge, normal.topEdge, normal.rightEdge, normal.bottomEdge,
					normal.mode, SRCCOPY);

			useTrueTransparent = normal.trueTransparent;
		}
		else if (buttonType == 1)
		{
			usedAlphaTransparency = active.alphaTransparency;
			usedBorderMethod = active.borderMethod;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryActiveDC, 0, 0, imageActiveWidth, imageActiveHeight,
					active.leftEdge, active.topEdge, active.rightEdge, active.bottomEdge,
					active.mode, SRCCOPY);

			useTrueTransparent = active.trueTransparent;
		}
		else if (buttonType == 2)
		{
			usedAlphaTransparency = minimized.alphaTransparency;
			usedBorderMethod = minimized.borderMethod;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryMinimizedDC, 0, 0, imageMinimizedWidth, imageMinimizedHeight,
					minimized.leftEdge, minimized.topEdge, minimized.rightEdge, minimized.bottomEdge,
					minimized.mode, SRCCOPY);

			useTrueTransparent = minimized.trueTransparent;
		}
		else if (buttonType == 3)
		{
			usedAlphaTransparency = flashing.alphaTransparency;
			usedBorderMethod = flashing.borderMethod;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryFlashingDC, 0, 0, imageFlashingWidth, imageFlashingHeight,
					flashing.leftEdge, flashing.topEdge, flashing.rightEdge, flashing.bottomEdge,
					flashing.mode, SRCCOPY);

			useTrueTransparent = flashing.trueTransparent;
		}
		else if (buttonType == 4)
		{
			usedAlphaTransparency = hover.alphaTransparency;
			usedBorderMethod = hover.borderMethod;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryHoverDC, 0, 0, imageHoverWidth, imageHoverHeight,
					hover.leftEdge, hover.topEdge, hover.rightEdge, hover.bottomEdge,
					hover.mode, SRCCOPY);

			useTrueTransparent = hover.trueTransparent;
		}
		else if (buttonType == 5)
		{
			usedAlphaTransparency = group.alphaTransparency;
			usedBorderMethod = group.borderMethod;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryGroupDC, 0, 0, imageGroupWidth, imageGroupHeight,
					group.leftEdge, group.topEdge, group.rightEdge, group.bottomEdge,
					group.mode, SRCCOPY);

			useTrueTransparent = group.trueTransparent;
		}
	}

	RECT completeFillRect = {0, 0, width, height};
	if (usedBorderMethod == 1)
		DrawEdge(bufferDC, &completeFillRect, EDGE_RAISED, BF_RECT | BF_SOFT);
	else if (usedBorderMethod == 2)
		DrawEdge(bufferDC, &completeFillRect, EDGE_SUNKEN, BF_RECT | BF_SOFT);
	else if (usedBorderMethod == 3)
		DrawEdge(bufferDC, &completeFillRect, EDGE_ETCHED, BF_RECT | BF_SOFT);
	else if (usedBorderMethod == 4)
		DrawEdge(bufferDC, &completeFillRect, EDGE_BUMP, BF_RECT | BF_SOFT);
	
	BITMAP bm;
	GetObject(bufferBitmap, sizeof(BITMAP), &bm);

	BYTE *pPixel = (LPBYTE)bm.bmBits;

	for (int j = 0; j < bm.bmHeight * bm.bmWidth; ++j)
	{
		//Remove Magic Pink
		if ( (pPixel[0] == 255 && pPixel[1] == 0 && pPixel[2] == 255) )
		{
			pPixel[0] = 0;
			pPixel[1] = 0;
			pPixel[2] = 0;
			pPixel[3] = 0;
		}
		else if ( !useAlphaMap || pPixel[3] == 0 )
		{
			pPixel[3] = 255;
		}

		pPixel+= 4;
	}
	
	BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, usedAlphaTransparency, AC_SRC_ALPHA };

	AlphaBlend(hDC, x, y, width, height, bufferDC, 0, 0, width, height, blendFunction);

	bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
	DeleteObject(bufferBitmap);
	DeleteDC(bufferDC);
}

void ButtonTexture::paintEffects(HDC destHDC, int width, int height, int button_saturation, int button_hueIntensity, COLORREF button_hueColor)
{
	//Paint Saturation and Hueing Effects in MemoryDC's
	int iy = 0;
	int ix = 0;
	COLORREF cl = 0;
	BYTE r = 0;
	BYTE g = 0;
	BYTE b = 0;

	for(iy = 0; iy < height; iy++)
	{
		for(ix = 0; ix < width; ix++)
		{
			cl = GetPixel(destHDC, ix, iy);
			r = GetRValue(cl);
			g = GetGValue(cl);
			b = GetBValue(cl);
		
			if( !( r == 255 && g == 0 && b == 255 ) )
			{
				// Saturation effect
				if(button_saturation != 255)
				{
					BYTE gray = (BYTE)(r * 0.3086 + g * 0.6094 + b * 0.0820);

					r = (BYTE)((r * button_saturation + gray * (255 - button_saturation) + 255) >> 8);
					g = (BYTE)((g * button_saturation + gray * (255 - button_saturation) + 255) >> 8);
					b = (BYTE)((b * button_saturation + gray * (255 - button_saturation) + 255) >> 8);
				}

				// Hue effect
				if(button_hueIntensity != 0)
				{
					r = (BYTE)((r * (255 - button_hueIntensity) + GetRValue(button_hueColor) * button_hueIntensity + 255) >> 8);
					g = (BYTE)((g * (255 - button_hueIntensity) + GetGValue(button_hueColor) * button_hueIntensity + 255) >> 8);
					b = (BYTE)((b * (255 - button_hueIntensity) + GetBValue(button_hueColor) * button_hueIntensity + 255) >> 8);
				}
			}
			else
			{
				transparencyNeeded = true;
			}
	
			SetPixelV(destHDC, ix, iy, RGB(r, g, b));
		}
	}
	//End Saturation / Hueing
}

void ButtonTexture::solidGradientPaint(HDC destHDC, ColorRefList workGradient, RECT r, int imageType)
{
	bool vertical = false;
	int repeated = 0;

	if ( imageType == 0 )
	{
		vertical = normalverticalGradient;
		repeated = normalrepeatedGradient;
	}
	else if( imageType == 1 )
	{
		vertical = activeverticalGradient;
		repeated = activerepeatedGradient;
	}
	else if( imageType == 2 )
	{
		vertical = minimizedverticalGradient;
		repeated = minimizedrepeatedGradient;
	}
	else if( imageType == 3 )
	{
		vertical = flashingverticalGradient;
		repeated = flashingrepeatedGradient;
	}
	else if( imageType == 4 )
	{
		vertical = hoververticalGradient;
		repeated = hoverrepeatedGradient;
	}
	else if( imageType == 5 )
	{
		vertical = groupverticalGradient;
		repeated = grouprepeatedGradient;
	}
	
	if ( repeated == 0 )
	{
		int nWidth = r.right-r.left;
		int nHeight = r.bottom-r.top;

		RECT r2 = {0,0,0,0};
		double cr, cg, cb, dr, dg, db;

		// ColorsSteps between Start and End Color
		int gradSteps = 0;
		int unequalRest = 0;

		int colorCount = workGradient.size();

		if (vertical)
		{
			gradSteps = (nHeight-colorCount)/(colorCount-1);
			unequalRest = (nHeight-colorCount)%(colorCount-1);
		}
		else
		{
			gradSteps = (nWidth-colorCount)/(colorCount-1);
			unequalRest = (nWidth-colorCount)%(colorCount-1);
		}

		int lastGradientPos = 0;
		if (vertical)
			lastGradientPos = r.top;
		else
			lastGradientPos = r.left;

		int counter = 0;
		for (ColorRefListIterator cit = workGradient.begin(); cit != workGradient.end(); cit++ )
		{
			if (cit == workGradient.end())
				break;

			counter++;

			COLORREF one = *cit;
			cit++;
			COLORREF two = *cit;
			cit--;
		
			// get current color
			cr = GetRValue( one );
			cg = GetGValue( one );
			cb = GetBValue( one );

			// Get delta color (+1 cause we must reach the Final Color)
			dr = (cr - GetRValue( two )) / (gradSteps+1);
			dg = (cg - GetGValue( two )) / (gradSteps+1);
			db = (cb - GetBValue( two )) / (gradSteps+1);

			//Paint Real Gradient Color
			//-------------------------------------------
			if ( unequalRest != 0 && counter > 1 )
			{
				unequalRest--;
				if (vertical)
				{
					SetRect(&r2, r.left, lastGradientPos, r.right, min(lastGradientPos+1, r.bottom));
				}
				else
				{
					SetRect(&r2, lastGradientPos, r.top, min(lastGradientPos+1, r.right), r.bottom);
				}

				if (r2.right-r2.left > 0 && r2.bottom-r2.top > 0)
				{
					::SetBkColor(destHDC, one);
					::ExtTextOut(destHDC, 0, 0, ETO_OPAQUE, &r2, NULL, 0, NULL);
				}
				
				if (vertical)
					lastGradientPos = r2.bottom;
				else
					lastGradientPos = r2.right;
			}
			//-------------------------------------------

			for (int i = 0; i <= gradSteps; i++)
			{
				if (vertical)
				{
					SetRect(&r2, r.left, lastGradientPos+i, r.right, min(lastGradientPos+i+1, r.bottom));
				}
				else
				{
					SetRect(&r2, lastGradientPos+i, r.top, min(lastGradientPos+i+1, r.right), r.bottom);
				}

				if (r2.right-r2.left > 0 && r2.bottom-r2.top > 0)
				{
					::SetBkColor(destHDC, RGB((int)cr, (int)cg, (int)cb));
					::ExtTextOut(destHDC, 0, 0, ETO_OPAQUE, &r2, NULL, 0, NULL);
				}
				
				cr -= dr;
				cg -= dg;
				cb -= db;
			}

			if (vertical)
				lastGradientPos = r2.bottom;
			else
				lastGradientPos = r2.right;
		}
	}
	else
	{
		int nWidth = r.right-r.left;
		int nHeight = r.bottom-r.top;

		RECT r2 = {0,0,0,0};
		COLORREF color;

		int lastGradientPos = 0;
		if (vertical)
			lastGradientPos = r.top;
		else
			lastGradientPos = r.left;

		int counter = 0;
		int nSize = 0;
		
		if (vertical)
			nSize = nHeight;
		else
			nSize = nWidth; 

		while ( counter < nSize )
		{
			for (ColorRefListIterator cit = workGradient.begin(); cit != workGradient.end(); cit++ )
			{
				color = *cit;

				if (vertical)
				{
					SetRect(&r2, r.left, lastGradientPos, r.right, min(lastGradientPos+repeated, r.bottom));
				}
				else
				{
					SetRect(&r2, lastGradientPos, r.top, min(lastGradientPos+repeated, r.right), r.bottom);
				}

				if (r2.right-r2.left > 0 && r2.bottom-r2.top > 0)
				{
					::SetBkColor(destHDC, color);
					::ExtTextOut(destHDC, 0, 0, ETO_OPAQUE, &r2, NULL, 0, NULL);
				}

				if (vertical)
					lastGradientPos = r2.bottom;
				else
					lastGradientPos = r2.right;

				counter++;

				if ( counter == nSize )
					break;
			}
		}
	}
}

void ButtonTexture::PrepareAlphaChannel(HBITMAP &bmp, int button_saturation, int button_hueIntensity, COLORREF button_hueColor)
{
	//Paint Saturation and Hueing Effects in MemoryDC's
	BITMAP bm;
	GetObject(bmp, sizeof(BITMAP), &bm);

	BYTE *pPixel = (LPBYTE)bm.bmBits;

	BYTE r = 0;
	BYTE g = 0;
	BYTE b = 0;

	for (int i = 0; i < bm.bmHeight * bm.bmWidth; ++i)
	{
		//Magic Pink
		if (pPixel[0] == 255 && pPixel[1] == 0 && pPixel[2] == 255)
			pPixel[3] = 0;

		//Prepare Text
		if (pPixel[3] == 0)
			pPixel[3] = 1;

		r = pPixel[0];
		g = pPixel[1];
		b = pPixel[2];

		//AlphaMap Colorizing
		//Saturation Effect
		if( button_saturation != 255 )
		{
			BYTE gray = (BYTE)(r * 0.3086 + g * 0.6094 + b * 0.0820);
			r = (BYTE)((r * button_saturation + gray * (255 - button_saturation) + 255) >> 8);
			g = (BYTE)((g * button_saturation + gray * (255 - button_saturation) + 255) >> 8);
			b = (BYTE)((b * button_saturation + gray * (255 - button_saturation) + 255) >> 8);
		}
		// Hue effect
		if( button_hueIntensity != 0 )
		{
			// The B & R values of the hue is swapped here, can somebody tell me why it only works that way?
			// test normal
			b = (BYTE)((r * (255 - button_hueIntensity) + GetRValue(button_hueColor) * button_hueIntensity + 255) >> 8);
			g = (BYTE)((g * (255 - button_hueIntensity) + GetGValue(button_hueColor) * button_hueIntensity + 255) >> 8);
			r = (BYTE)((b * (255 - button_hueIntensity) + GetBValue(button_hueColor) * button_hueIntensity + 255) >> 8);
		}
		
		//PreMultiply
		pPixel[0] = r * pPixel[3] / 255;
		pPixel[1] = g * pPixel[3] / 255;
		pPixel[2] = b * pPixel[3] / 255;

		pPixel+= 4;
	}
}