/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2005 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#if !defined(AFX_BUTTONTEXTURE_H__4F10FDCD_EF77_423F_8223_011704DFFED2__INCLUDED_)
#define AFX_BUTTONTEXTURE_H__4F10FDCD_EF77_423F_8223_011704DFFED2__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

typedef vector<COLORREF> ColorRefList;
typedef vector<COLORREF>::iterator ColorRefListIterator;

struct BUTTONSETTINGS
{
	int borderMethod;

	int leftEdge;
	int topEdge;
	int rightEdge;
	int bottomEdge;

	int mode;

	int button_saturation;
	int button_hueIntensity;
	COLORREF button_hueColor;

	bool trueTransparent;

	int alphaTransparency;
};

class ButtonTexture  
{
public:

	ButtonTexture(const string &name);
	virtual ~ButtonTexture();

	void configure(const string &name, bool alphaMap);
	void apply(HDC hDC, int x, int y, int width, int height, int buttonType);
	void applyOverlay(HDC hDC, int x, int y, int width, int height, int buttonType);

	bool isTransparencyNeeded(){ return transparencyNeeded; };

	bool useHoverBG() { return (usehoverSolidColors || (memoryHoverDC != 0 && memoryHoverBitmap != 0)); };
	bool useGroupedBG() { return (usegroupSolidColors || (memoryGroupDC != 0 && memoryGroupBitmap != 0)); };

	bool useAlphaTransparency(int buttonType);

private:

	bool transparencyNeeded;

	bool useAlphaMap;

	void subconfigure(string type, BUTTONSETTINGS &workSettings, BUTTONSETTINGS &defaultSettings);

	void parseCFG(string cfg, COLORREF &bgcolor, COLORREF &lightcolor, COLORREF &darkcolor, int imageType);
	void parseGradientCFG(string cfg, int imageType);
	void parseBevelCFG(string cfg, int &leftBevel, int &rightBevel, int &topBevel, int &bottomBevel);
	void solidGradientPaint(HDC destHDC, ColorRefList workGradient, RECT r, int imageType);
	void paintEffects(HDC destHDC, int width, int height, int button_saturation, int button_hueIntensity, COLORREF button_hueColor );
	void PrepareAlphaChannel(HBITMAP &bmp, int button_saturation, int button_hueIntensity, COLORREF button_hueColor);

	ColorRefList NormalGradient;
	ColorRefList ActiveGradient;
	ColorRefList MinimizedGradient;
	ColorRefList FlashingGradient;

	ColorRefList HoverGradient;
	ColorRefList GroupGradient;

	bool normalContainsAlpha;
	bool activeContainsAlpha;
	bool minimizedContainsAlpha;
	bool flashingContainsAlpha;

	bool hoverContainsAlpha;
	bool groupContainsAlpha;

	HDC memoryNormalDC;
	HBITMAP memoryNormalBitmap;

	HDC memoryActiveDC;
	HBITMAP memoryActiveBitmap;

	HDC memoryMinimizedDC;
	HBITMAP memoryMinimizedBitmap;

	HDC memoryFlashingDC;
	HBITMAP memoryFlashingBitmap;

	HDC memoryHoverDC;
	HBITMAP memoryHoverBitmap;

	HDC memoryGroupDC;
	HBITMAP memoryGroupBitmap;

	int imageNormalHeight;
	int imageNormalWidth;

	int imageActiveHeight;
	int imageActiveWidth;

	int imageMinimizedHeight;
	int imageMinimizedWidth;

	int imageFlashingHeight;
	int imageFlashingWidth;

	int imageHoverHeight;
	int imageHoverWidth;

	int imageGroupHeight;
	int imageGroupWidth;

	BUTTONSETTINGS defaultbutton;
	BUTTONSETTINGS normal;
	BUTTONSETTINGS active;
	BUTTONSETTINGS minimized;
	BUTTONSETTINGS flashing;
	BUTTONSETTINGS hover;	
	BUTTONSETTINGS group;

	string _defaultsolidcolors;
	string _defaultgradientcolor;
	string _defaultbevelsize;
	bool _defaultverticalgradient;
	int _defaultrepeatedgradient;

	string _normalsolidcolors;
	string _normalbevelsize;
	string _normalgradientcolor;

	bool usenormalSolidColors;
	bool normalverticalGradient;
	int normalrepeatedGradient;
	int normalbevelSizeLeft, normalbevelSizeRight, normalbevelSizeTop, normalbevelSizeBottom;
	COLORREF normalbackgroundColor;
	COLORREF normaldarkBevelColor;
	COLORREF normallightBevelColor;

	string _activesolidcolors;
	string _activegradientcolor;

	bool useactiveSolidColors;
	bool activeverticalGradient;
	int activerepeatedGradient;
	int activebevelSizeLeft, activebevelSizeRight, activebevelSizeTop, activebevelSizeBottom;
	COLORREF activebackgroundColor;
	COLORREF activedarkBevelColor;
	COLORREF activelightBevelColor;

	bool useminimizedSolidColors;
	bool minimizedverticalGradient;
	int minimizedrepeatedGradient;
	int minimizedbevelSizeLeft, minimizedbevelSizeRight, minimizedbevelSizeTop, minimizedbevelSizeBottom;
	COLORREF minimizedbackgroundColor;
	COLORREF minimizeddarkBevelColor;
	COLORREF minimizedlightBevelColor;

	bool useflashingSolidColors;
	bool flashingverticalGradient;
	int flashingrepeatedGradient;
	int flashingbevelSizeLeft, flashingbevelSizeRight, flashingbevelSizeTop, flashingbevelSizeBottom;
	COLORREF flashingbackgroundColor;
	COLORREF flashingdarkBevelColor;
	COLORREF flashinglightBevelColor;

	bool usehoverSolidColors;
	bool hoververticalGradient;
	int hoverrepeatedGradient;
	int hoverbevelSizeLeft, hoverbevelSizeRight, hoverbevelSizeTop, hoverbevelSizeBottom;
	COLORREF hoverbackgroundColor;
	COLORREF hoverdarkBevelColor;
	COLORREF hoverlightBevelColor;

	bool usegroupSolidColors;
	bool groupverticalGradient;
	int grouprepeatedGradient;
	int groupbevelSizeLeft, groupbevelSizeRight, groupbevelSizeTop, groupbevelSizeBottom;
	COLORREF groupbackgroundColor;
	COLORREF groupdarkBevelColor;
	COLORREF grouplightBevelColor;
};

#endif // !defined(AFX_BUTTONTEXTURE_H__4F10FDCD_EF77_423F_8223_011704DFFED2__INCLUDED_)
