/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2005 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "Utilities.h"

#define INVALID_ICON(x) ((NULL == (x)) || (1 == (int)(x)) || (-1 == (int)(x)))

HDC hdcDesktop = 0;
HBITMAP hbmDesktop = 0;

extern int realMonitors;
extern int monitorIndex;
extern MONITORINFO monitorInfoArray[3];
extern SIZE monitorSizeArray[3];

extern int minscreenX;
extern int minscreenY;
extern int maxscreenWidth;
extern int maxscreenHeight;

BOOL CALLBACK EnumMonitorsProc(HMONITOR hMonitor, HDC hdc, LPRECT lpRMonitor, LPARAM dwData)
{
	MONITORINFO mi = { sizeof(MONITORINFO) };
	GetMonitorInfo(hMonitor, &mi);
	monitorInfoArray[monitorIndex] = mi;
	monitorSizeArray[monitorIndex].cx = mi.rcMonitor.right-mi.rcMonitor.left;
	monitorSizeArray[monitorIndex].cy = mi.rcMonitor.bottom-mi.rcMonitor.top;
	monitorIndex++;

	if ( pMon == NULL && mi.dwFlags == MONITORINFOF_PRIMARY )
		pMon = hMonitor;
	else if ( sMon == NULL && mi.dwFlags != MONITORINFOF_PRIMARY )
		sMon = hMonitor;
	else if ( tMon == NULL && mi.dwFlags != MONITORINFOF_PRIMARY )
		tMon = hMonitor;

	return true;
}

void getMonitorInfo()
{
	realMonitors = GetSystemMetrics(SM_CMONITORS);
	monitorIndex = 0;
	EnumDisplayMonitors(NULL, NULL, EnumMonitorsProc, 0);
}

string GetTaskClass( HWND hTask )
{
	char windowClass[MAX_PATH_LENGTH] = {0};

	if (hTask)
		GetClassName( hTask, windowClass, MAX_PATH_LENGTH );

	return (string) windowClass;
}

string GetTaskWindowText( HWND hTask )
{
	char windowText[MAX_PATH_LENGTH] = {0};

	if (hTask)
		GetWindowText( hTask, windowText, MAX_PATH_LENGTH );

	return (string) windowText;
}

string GetTaskFileName( HWND hTask )
{
	HINSTANCE hLib = LoadLibraryA("PSAPI.DLL");
	if (!hLib) 
		return GetTaskClass( hTask );

	BOOL (WINAPI *lpfEnumProcessModules)( HANDLE, HMODULE *, DWORD, LPDWORD );
	DWORD (WINAPI *lpfGetModuleFileNameEx)( HANDLE, HMODULE, LPTSTR, DWORD );

	lpfEnumProcessModules = (BOOL(WINAPI *)(HANDLE, HMODULE *, DWORD, LPDWORD)) GetProcAddress( hLib, "EnumProcessModules" ) ;
	lpfGetModuleFileNameEx =(DWORD (WINAPI *)(HANDLE, HMODULE, LPTSTR, DWORD )) GetProcAddress( hLib, "GetModuleFileNameExA" ) ;
	
	if( lpfEnumProcessModules == NULL || lpfGetModuleFileNameEx == NULL )
	{
		// some of the functions did not load properly... return out
		FreeLibrary(hLib);
		return GetTaskClass( hTask );
	}
	else
	{
		// get the exe names
		HANDLE hProcess;
		HMODULE hModule;
		DWORD dwSize2;
		char szFilename[MAX_PATH_LENGTH] = {0};

		// open the process so we can get a handle
		DWORD dwPID;
		GetWindowThreadProcessId(hTask, &dwPID);
		hProcess = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, false, dwPID);
		if (hProcess)
		{
			if (lpfEnumProcessModules(hProcess, &hModule, sizeof(hModule), &dwSize2))
			{
				// get path name
				if (!lpfGetModuleFileNameEx(hProcess, hModule, szFilename, sizeof(szFilename)))
				{
					CloseHandle(hProcess); // be nice. :)
					return GetTaskClass( hTask );
				}
			}
			CloseHandle(hProcess); // be nice. :)
			
			return (string) szFilename;
		}
		
		return GetTaskClass( hTask );
	}
}

void PrefixedLSSetVariable(string prefix, string var, string value)
{
	if ( !var.empty() )
		prefix.append(var);

	LSSetVariable(prefix.c_str(), value.c_str());
}

StringList split(string str, string sep)
{
	StringList substrs;
	substrs.clear();

	if ( str.empty() )
		return substrs;

	int start = 0;
	int end = str.find(sep, start);
	
	while(end != string::npos)
	{
		substrs.push_back(str.substr(start, end - start - sep.length() + 1));
		start = end + sep.length();
		end = str.find(sep, start);
	}
	
	substrs.push_back(str.substr(start));

	return substrs;
}

bool IsSubString(string big, string small)
{
	if( match( small.c_str(), big.c_str() ) )
		return TRUE;
	else
		return FALSE;
}

bool IsOnTaskbarList(HWND hWnd, StringList theList, bool emptyValue)
{	
	//Matching Lists
	if ( theList.empty() )
		return emptyValue;

	string tmpTitle = GetTaskWindowText( hWnd );
	string tmpClass = GetTaskClass( hWnd );
	
	for (StringListIterator it = theList.begin(); it != theList.end(); ++it)
	{
		if (IsSubString(tmpTitle, (*it).c_str()) || IsSubString(tmpClass, (*it).c_str()))
			return TRUE;
	}
	return FALSE;
}

void GetTaskIcon(HWND hTask, HICON &hIcon, bool bBigIcon)
{
	int nTimeOut = 1000;

	if(bBigIcon)
	{
		if(SendMessageTimeout(	 hTask
								,WM_GETICON
								,ICON_BIG
								,0
								,/*SMTO_BLOCK|*/SMTO_ABORTIFHUNG
								,nTimeOut
								,(LPDWORD)&hIcon
								) && !INVALID_ICON(hIcon) )
			return;

		hIcon = (HICON)GetClassLong(hTask, GCL_HICON);
		if(!INVALID_ICON(hIcon))
			return;

		if(SendMessageTimeout(	 hTask
								,WM_QUERYDRAGICON
								,ICON_BIG
								,0
								,/*SMTO_BLOCK|*/SMTO_ABORTIFHUNG
								,nTimeOut
								,(LPDWORD)&hIcon
							) && !INVALID_ICON(hIcon))
			return;

		if(SendMessageTimeout(	 hTask
							,WM_GETICON
							,ICON_SMALL
							,0
							,/*SMTO_BLOCK|*/SMTO_ABORTIFHUNG
							,nTimeOut
							,(LPDWORD)&hIcon
						) && !INVALID_ICON(hIcon))
			return;

		hIcon = (HICON)GetClassLong(hTask, GCL_HICONSM);
		if(!INVALID_ICON(hIcon))
			return;

		if(SendMessageTimeout(	 hTask
								,WM_QUERYDRAGICON
								,ICON_SMALL
								,0
								,/*SMTO_BLOCK|*/SMTO_ABORTIFHUNG
								,nTimeOut
								,(LPDWORD)&hIcon
							) && !INVALID_ICON(hIcon))
			return;	
	}
	else
	{
		if(SendMessageTimeout(	 hTask
								,WM_GETICON
								,ICON_SMALL
								,0
								,/*SMTO_BLOCK|*/SMTO_ABORTIFHUNG
								,nTimeOut
								,(LPDWORD)&hIcon
							) && !INVALID_ICON(hIcon))
			return;

		hIcon = (HICON)GetClassLong(hTask, GCL_HICONSM);
		if(!INVALID_ICON(hIcon))
			return;

		if(SendMessageTimeout(	 hTask
								,WM_QUERYDRAGICON
								,ICON_SMALL
								,0
								,/*SMTO_BLOCK|*/SMTO_ABORTIFHUNG
								,nTimeOut
								,(LPDWORD)&hIcon
							) && !INVALID_ICON(hIcon))
			return;
		
		if(SendMessageTimeout(	 hTask
								,WM_GETICON
								,ICON_BIG
								,0
								,/*SMTO_BLOCK|*/SMTO_ABORTIFHUNG
								,nTimeOut
								,(LPDWORD)&hIcon
								) && !INVALID_ICON(hIcon) )
			return;

		hIcon = (HICON)GetClassLong(hTask, GCL_HICON);
		if(!INVALID_ICON(hIcon))
			return;

		if(SendMessageTimeout(	 hTask
								,WM_QUERYDRAGICON
								,ICON_BIG
								,0
								,/*SMTO_BLOCK|*/SMTO_ABORTIFHUNG
								,nTimeOut
								,(LPDWORD)&hIcon
							) && !INVALID_ICON(hIcon))
			return;
	}

	//All Failed
	hIcon = LoadIcon(NULL, MAKEINTRESOURCE(IDI_APPLICATION));
}

int ParseIconLocation(LPSTR szIconFile)
{
	LPSTR pszComma = strrchr(szIconFile, ',');

	if (!pszComma)
		return -1;

	*pszComma = 0;
	return atoi(pszComma + 1);
}

bool GetIconOnCustomList(HWND hWnd, HICON &hIcon, StringList theList, bool bBigIcon)
{
	if ( !theList.empty() )
	{
		string tmpTitle = GetTaskWindowText( hWnd );
		string tmpClass = GetTaskClass( hWnd );

		for (StringListIterator it = theList.begin(); it != theList.end(); ++it)
		{
			char search[64], iconPath[MAX_PATH_LENGTH];
			char *buffers[] = {search, iconPath};

			LCTokenize((*it).c_str(), buffers, 2, NULL);
			
			if (IsSubString(tmpTitle, search) || IsSubString(tmpClass, search))
			{
				int nIconIndex = ParseIconLocation(iconPath);

				if (nIconIndex >= 0)
				{
					ExtractIconEx(iconPath, nIconIndex,	(!bBigIcon ? NULL : &hIcon), (!bBigIcon ? &hIcon : NULL), 1);
				}
				else
				{
					SHFILEINFO shFileInfo;

					SHGetFileInfo(iconPath, NULL, &shFileInfo, sizeof(shFileInfo), SHGFI_ICON | (bBigIcon ? SHGFI_LARGEICON : SHGFI_SMALLICON) );

					hIcon = shFileInfo.hIcon;
				}
				return false;
			}
		}
	}

	GetTaskIcon(hWnd, hIcon, bBigIcon);
	return true;
}

bool IsOS(const int &os)
{	
	static int osversion = 0;

	static bool bInitialized = false;

	if (!bInitialized)
	{
		OSVERSIONINFO verInfo;
		verInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
		GetVersionEx(&verInfo);
		
		switch(verInfo.dwPlatformId)
		{
			case VER_PLATFORM_WIN32_WINDOWS:

				osversion |= OS_9X;
			
				if(verInfo.dwMinorVersion >= 90)
				{
					osversion |= OS_ME;
				}
				else if(verInfo.dwMinorVersion >= 10)
				{
					osversion |= OS_98;
				}
				else
				{
					osversion |= OS_95;
				}
				break;
			
			case VER_PLATFORM_WIN32_NT:
			
				osversion |= OS_NT;
			
				if(verInfo.dwMajorVersion >= 5)
				{
					osversion |= OS_2KXP;

					if (verInfo.dwMinorVersion >= 1)
					{
						osversion |= OS_XP;
					}
					else
					{
						osversion |= OS_2K;
					}
				}
				else
				{
					osversion |= OS_NT4;
				}
				break;
			default:;
		}
		bInitialized = true;
	}

	return (osversion & os) ? true : false;
}

string QueryPixelDepth()
{
	CString Response = "";
	DEVMODE DevMode;
	DevMode.dmSize=sizeof(DEVMODE);
	
	if (EnumDisplaySettings(NULL,ENUM_CURRENT_SETTINGS,&DevMode))
	{
		Response.Format("%ld", DevMode.dmBitsPerPel);
	}
	else
	{
		Response = "8";
	}
	
	return (string)Response;
}

// retrieve handle of the desktop window
HWND GetLitestepDesktop()
{
	HWND hWnd = FindWindow("DesktopBackgroundClass", NULL);

	if(hWnd != 0)
		return hWnd;

	return GetDesktopWindow();
}

HWND FindWindowEx2(string pszClass, string pszTitle)
{
	HWND hWnd = NULL;
	FindWindowEx2Data fwed;
	fwed.phWnd = &hWnd;
	fwed.pszClass = pszClass.c_str();
	fwed.pszTitle = pszTitle.c_str();
	EnumWindows(FindWindowEx2EnumWindowsProc, (LPARAM)&fwed);
	return *(fwed.phWnd);
}

BOOL CALLBACK FindWindowEx2EnumWindowsProc(HWND hWnd, LPARAM lParam)
{
	FindWindowEx2Data* pfwed = (FindWindowEx2Data*)lParam;
	char szClass[1024];
	char szTitle[1024];
	if(pfwed->pszClass)
	{
		GetClassName(hWnd, szClass, 1024);
	}
	if(pfwed->pszTitle)
	{
		GetWindowText(hWnd, szTitle, 1024);
	}
	if((pfwed->pszClass != NULL ? !stricmp(szClass, pfwed->pszClass) : TRUE) && (pfwed->pszTitle != NULL ? !stricmp(szTitle, pfwed->pszTitle) : TRUE))
	{
		*(pfwed->phWnd) = hWnd;
		return FALSE;
	}
	EnumChildWindows(hWnd, FindWindowEx2EnumWindowsProc, lParam);

	return (*pfwed->phWnd == NULL);
}

// is the given window a toplevel application window?
bool IsAppWindow(const HWND &hWnd)
{
	if (!IsWindow(hWnd))
		return false;
	
	if (!IsWindowVisible(hWnd))
		return false;

	if ( GetWindowLong(hWnd, GWL_USERDATA) == magicDWord )
		return false;

	long exStyle = GetWindowLong(hWnd, GWL_EXSTYLE);

	if ( exStyle & WS_EX_APPWINDOW )
		return true;
	
	if ( GetParent( hWnd ) )
		return false;
	
	if ( GetWindow(hWnd, GW_OWNER) )
		return false;

	if ( exStyle & WS_EX_TOOLWINDOW )
		return false;

	return true;
}

bool IsValidWindow(const HWND &hWnd)
{
	/* Ask VWM if the hWnd is on the current desktop */
	LRESULT lResult = SendMessage(GetLitestepWnd(), LM_GETDESKTOPOF, (WPARAM)hWnd, 0x00000001);
	/*
	 * Some VWMs return -1 for any window that it doesn't 'manage', so we should accept those
	 * windows as always being on the current desktop.  Otherwise, we need to check the 'active'
	 * bit in the uppper nible of the return value.  If it is set, then the window was on the
	 * current desktop.
	*/
	if (0 > lResult || 0x01000000 == (lResult & 0xFF000000))
		return true;
	else 
		return false;
}

// modify window style
void ModifyStyle(HWND hWnd, DWORD removeStyle, DWORD addStyle)
{
	SetWindowLong(hWnd, GWL_STYLE, (GetWindowLong(hWnd, GWL_STYLE) & ~removeStyle) | addStyle);
}

void ModifyStyleEx(HWND hWnd, DWORD removeStyle, DWORD addStyle)
{
	SetWindowLong(hWnd, GWL_EXSTYLE, (GetWindowLong(hWnd, GWL_EXSTYLE) & ~removeStyle) | addStyle);
}

int SplitString(const string &process, string &one, string &two, string &three, string &four, string &five, string &six, string &seven)
{
	int count = 0;
	one = "";
	two = "";
	three = "";
	four = "";
	five = "";
	six = "";
	seven = "";
	char cone[256] = {0};
	char ctwo[256] = {0};
	char cthree[256] = {0};
	char cfour[256] = {0};
	char cfive[256] = {0};
	char csix[256] = {0};
	char cseven[256] = {0};
	char *cbuffers[] = {cone, ctwo, cthree, cfour, cfive, csix, cseven};
		
	int numTokens = LCTokenize( process.c_str(), cbuffers, 7, NULL);
	if( numTokens >= 1 )
	{
		one = cone;
		count++;
		
		if ( numTokens >= 2 )
		{
			two = ctwo;
			count++;
		}
		
		if ( numTokens >= 3 )
		{
			three = cthree;
			count++;
		}
		
		if ( numTokens >= 4 )
		{
			four = cfour;
			count++;
		}

		if ( numTokens >= 5 )
		{
			five = cfive;
			count++;
		}

		if ( numTokens >= 6 )
		{
			six = csix;
			count++;
		}

		if ( numTokens >= 7 )
		{
			seven = cseven;
			count++;
		}
	}

	return count;
}

// trim leading and trailing whitespace
string trim(const string &toTrim)
{
	int start = 0;
	int end = toTrim.length() - 1;
	
	while(start <= end && isspace(toTrim[start]))
		start++;
	
	while(start <= end && isspace(toTrim[end]))
		end--;
	
	return toTrim.substr(start, end - start + 1);
}

void GetxRCList(char *text, StringList &theList, bool trim)
{
	FILE * f = LCOpen(NULL);

	if(f != NULL)
	{
		char lineBuffer[MAX_LINE_LENGTH] = {0};

		while(LCReadNextConfig(f, text, lineBuffer, MAX_LINE_LENGTH))
		{
			char temp[64], configline[MAX_LINE_LENGTH];
			char *buffers[] = {temp};
			
			if(LCTokenize(lineBuffer, buffers, 1, configline) >= 1)
			{
				CString worker = configline;
				if ( trim )
					worker.Replace("\"", "");

				theList.insert(theList.end(), (string) worker);
			}
		}
		
		LCClose(f);
	}
}

StringList GetxRCLineList(const string &uniqueKey)
{
	StringList resultList;
	FILE * f = LCOpen(NULL);

	if(f != NULL)
	{
		char lineBuffer[MAX_LINE_LENGTH];

		while(LCReadNextConfig(f, uniqueKey.c_str(), lineBuffer, MAX_LINE_LENGTH))
		{
			char temp[64], configline[MAX_LINE_LENGTH];
			char *buffers[] = {temp};
			
			if(LCTokenize(lineBuffer, buffers, 1, configline) >= 1)
			{
				resultList.insert(resultList.end(), configline);
			}
		}
		
		LCClose(f);
	}

	return resultList;
}


// retrieve a boolean value from the step.rc
bool GetxRCBool(const string &uniqueKey, const string &baseName, bool defaultVal)
{
	string key = uniqueKey + baseName;
	return GetRCBoolDef(key.c_str(), defaultVal) ? true : false;
}

// retrieve a boolean value from the step.rc
bool Getx2RCBool(const string &uniqueKey, const string &groupKey, const string &baseName, bool defaultVal)
{
	return GetxRCBool(uniqueKey, baseName, GetxRCBool(groupKey, baseName, GetxRCBool("xModules", baseName, defaultVal)));
}

// retrieve a string value from step.rc
string GetxRCString(const string &uniqueKey, const string &baseName, const string &defaultVal)
{
	string key = uniqueKey + baseName;
	char buffer[MAX_LINE_LENGTH];
	GetRCString(key.c_str(), buffer, defaultVal.c_str(), MAX_LINE_LENGTH);
	return string(buffer);
}

// retrieve a string value from step.rc
string Getx2RCString(const string &uniqueKey, const string &groupKey, const string &baseName, const string &defaultVal)
{
	return GetxRCString(uniqueKey, baseName, GetxRCString(groupKey, baseName, GetxRCString("xModules", baseName, defaultVal)));
}

// retrieve a line from step.rc
string GetxRCLine(const string &uniqueKey, const string &baseName, const string &defaultVal)
{
	string key = uniqueKey + baseName;
	char buffer[MAX_LINE_LENGTH];
	GetRCLine(key.c_str(), buffer, MAX_LINE_LENGTH, defaultVal.c_str());

	CString worker = buffer;
	if ( worker.Left(1) == '"' )
		worker = worker.Right(worker.GetLength()-1);
	if ( worker.Right(1) == '"' )
		worker = worker.Left(worker.GetLength()-1);

	return (string) worker;
}

// retrieve a line from step.rc
string Getx2RCLine(const string &uniqueKey, const string &groupKey, const string &baseName, const string &defaultVal)
{
	return GetxRCLine(uniqueKey, baseName, GetxRCLine(groupKey, baseName, GetxRCLine("xModules", baseName, defaultVal)));
}

// retrieve a named value from step.rc
int GetxRCNamedValue(const string &uniqueKey, const string &baseName, const NameValuePair *nameValuePairs, int defaultVal)
{
	string name = GetxRCString(uniqueKey, baseName, "");
	const NameValuePair *current = nameValuePairs;

	if(name.length() > 0)
	{
		while(current->name)
		{
			if(stricmp(current->name, name.c_str()) == 0)
				return current->value;

			current++;
		}
	}

	return defaultVal;
}

int Getx2RCNamedValue(const string &uniqueKey, const string &groupKey, const string &baseName, const NameValuePair *nameValuePairs, int defaultVal)
{
	return GetxRCNamedValue(uniqueKey, baseName, nameValuePairs, GetxRCNamedValue(groupKey, baseName, nameValuePairs, GetxRCNamedValue("xModules", baseName, nameValuePairs, defaultVal)));
}

// retrieve a color value from step.rc
int GetxRCColor(const string &uniqueKey, const string &baseName, int defaultVal)
{
	string key = uniqueKey + baseName;
	return GetRCColor(key.c_str(), defaultVal);
}

// retrieve a color value from step.rc
int Getx2RCColor(const string &uniqueKey, const string &groupKey, const string &baseName, int defaultVal)
{
	return GetxRCColor(uniqueKey, baseName, GetxRCColor(groupKey, baseName, GetxRCColor("xModules", baseName, defaultVal)));
}

// retrieve an integer value from step.rc, make sure it is between minVal and maxVal inclusive
int GetxRCInt(const string &uniqueKey, const string &baseName, int defaultVal, int minVal, int maxVal)
{
	string key = uniqueKey + baseName;
	int answer = GetRCInt(key.c_str(), defaultVal);
	return max(min(answer, maxVal), minVal);
}

// retrieve an integer value from step.rc, make sure it is between minVal and maxVal inclusive
int Getx2RCInt(const string &uniqueKey, const string &groupKey, const string &baseName, int defaultVal, int minVal, int maxVal)
{
	return GetxRCInt(uniqueKey, baseName, GetxRCInt(groupKey, baseName, GetxRCInt("xModules" , baseName, defaultVal, minVal, maxVal), minVal, maxVal), minVal, maxVal);
}

// parse a coordinate specification
int xParseCoord(const string &aString, int defaultVal, int maxVal, HWND box, bool vertical, bool alwaysontop)
{
	string strVal = trim(aString);

	if (strVal.length() == 0)
		return defaultVal;

	bool negative = false;
	bool realnegative = false;
	bool center = false;
	UINT i = 0;
	int value = 0;

	if ( realMonitors > 1 && box == NULL )
	{
		int minx = 0;
		int miny = 0;
		int maxwidth = maxVal;
		int maxheight = maxVal;
		if ( strstr(strVal.c_str(), "@") != NULL && (strVal[i] == 'v' || strVal[i] == 'V') && realMonitors >= 2 )
		{
			if ( alwaysontop )
			{
				minx = minscreenX;
				miny = minscreenY;
			}
			else
			{
				minx = 0;
				miny = 0;
			}
			maxwidth = maxscreenWidth;
			maxheight = maxscreenHeight;
		}
		else if ( strstr(strVal.c_str(), "@") != NULL && strVal[i] == '2' && realMonitors >= 2 )
		{
			for (int x = 0; x < monitorIndex; x++)
			{
				if ( monitorInfoArray[x].dwFlags != MONITORINFOF_PRIMARY )
				{
					if ( alwaysontop )
					{
						minx = monitorInfoArray[x].rcMonitor.left;
						miny = monitorInfoArray[x].rcMonitor.top;
					}
					else
					{
						minx = monitorInfoArray[x].rcMonitor.left-minscreenX;
						miny = monitorInfoArray[x].rcMonitor.top-minscreenY;
					}
					maxwidth = monitorSizeArray[x].cx;
					maxheight = monitorSizeArray[x].cy;

					break;
				}
			}
		}
		else if ( strstr(strVal.c_str(), "@") != NULL && strVal[i] == '3' && realMonitors >= 3 )
		{
			int skip = 1;
			for (int x = 0; x < monitorIndex; x++)
			{
				if ( monitorInfoArray[x].dwFlags != MONITORINFOF_PRIMARY )
				{
					if ( skip > 0 )
					{
						skip--;
						continue;
					}
					if ( alwaysontop )
					{
						minx = monitorInfoArray[x].rcMonitor.left;
						miny = monitorInfoArray[x].rcMonitor.top;
					}
					else
					{
						minx = monitorInfoArray[x].rcMonitor.left-minscreenX;
						miny = monitorInfoArray[x].rcMonitor.top-minscreenY;
					}
					maxwidth = monitorSizeArray[x].cx;
					maxheight = monitorSizeArray[x].cy;

					break;
				}
			}
		}
		else //if ( strVal[i] == '1' )
		{
			for (int x = 0; x < monitorIndex; x++)
			{
				if ( monitorInfoArray[x].dwFlags == MONITORINFOF_PRIMARY )
				{
					if ( alwaysontop )
					{
						minx = monitorInfoArray[x].rcMonitor.left;
						miny = monitorInfoArray[x].rcMonitor.top;
					}
					else
					{
						minx = monitorInfoArray[x].rcMonitor.left-minscreenX;
						miny = monitorInfoArray[x].rcMonitor.top-minscreenY;
					}
					maxwidth = monitorSizeArray[x].cx;
					maxheight = monitorSizeArray[x].cy;

					break;
				}
			}
		}
		
		if ( strstr(strVal.c_str(), "@") != NULL )
			i = i+2;

		if (strVal[i] == '-')
		{
			negative = true;
			i++;
		}
		else if (strVal[i] == '~')
		{
			realnegative = true;
			i++;
		}
		else if (strVal[i] == '+')
		{
			i++;
		}

		while (i < strVal.length())
		{
			if(strVal[i] >= '0' && strVal[i] <= '9')
			{
				value = (value * 10) + (strVal[i] - '0');
			}
			else
			{
				if(strVal[i] == 'c' || strVal[i] == 'C')
					center = true;

				break;
			}

			i++;
		}

		if (center)
		{
			if (realnegative)
				negative = true;

			if (negative)
				value = (vertical ? miny+(maxheight/2) : minx+(maxwidth/2)) - value;
			else
				value = (vertical ? miny+(maxheight/2) : minx+(maxwidth/2)) + value;
		}
		else if (realnegative)
		{
			value = (vertical ? miny : minx) - value;
		}
		else if (negative)
		{
			value = (vertical ? miny+maxheight : minx+maxwidth) - value;
		}
		else
		{
			value = (vertical ? miny : minx) + value;
		}
	}
	else
	{
		if ( strstr(strVal.c_str(), "@") != NULL )
			i = i+2;

		if (strVal[i] == '-')
		{
			negative = true;
			i++;
		}
		else if (strVal[i] == '~')
		{
			realnegative = true;
			i++;
		}
		else if (strVal[i] == '+')
		{
			i++;
		}

		while (i < strVal.length())
		{
			if(strVal[i] >= '0' && strVal[i] <= '9')
			{
				value = (value * 10) + (strVal[i] - '0');
			}
			else
			{
				if(strVal[i] == 'c' || strVal[i] == 'C')
					center = true;

				break;
			}

			i++;
		}

		if (center)
		{
			if (realnegative)
				negative = true;

			if (negative)
				value = (maxVal/2) - value;
			else
				value = (maxVal/2) + value;
		}
		else if (realnegative)
		{
			value = -value;
		}
		else if (negative)
		{
			value = maxVal - value;
		}
	}

	return value;
}

// retrieve a coordinate from step.rc, accounting for negative values and centering
int GetxRCCoord(const string &uniqueKey, const string &baseName, int defaultVal, int maxVal, HWND box, bool vertical, bool alwaysontop)
{
	string strVal = GetxRCString(uniqueKey, baseName, "");
	return xParseCoord(strVal, defaultVal, maxVal, box, vertical, alwaysontop);
}

// parse a dimension
int xParseDimen(const string &aString, int defaultVal, int maxVal, HWND box, bool vertical)
{
	string strVal = trim(aString);

	if (strVal.length() == 0)
		return defaultVal;

	bool negative = false;
	bool percentage = false;
	UINT i = 0;
	int value = 0;

	if ( realMonitors > 1 && box == NULL )
	{
		int maxwidth = maxVal;
		int maxheight = maxVal;
		if ( strstr(strVal.c_str(), "@") != NULL && (strVal[i] == 'v' || strVal[i] == 'V') && realMonitors >= 2 )
		{
			maxwidth = maxscreenWidth;
			maxheight = maxscreenHeight;
		}
		else if ( strstr(strVal.c_str(), "@") != NULL && strVal[i] == '2' && realMonitors >= 2 )
		{
			for (int x = 0; x < monitorIndex; x++)
			{
				if ( monitorInfoArray[x].dwFlags != MONITORINFOF_PRIMARY )
				{
					maxwidth = monitorSizeArray[x].cx;
					maxheight = monitorSizeArray[x].cy;

					break;
				}
			}
		}
		else if ( strstr(strVal.c_str(), "@") != NULL && strVal[i] == '3' && realMonitors >= 3 )
		{
			int skip = 1;
			for (int x = 0; x < monitorIndex; x++)
			{
				if ( monitorInfoArray[x].dwFlags != MONITORINFOF_PRIMARY )
				{
					if ( skip > 0 )
					{
						skip--;
						continue;
					}
					maxwidth = monitorSizeArray[x].cx;
					maxheight = monitorSizeArray[x].cy;

					break;
				}
			}
		}
		else //if ( strVal[i] == '1' )
		{
			for (int x = 0; x < monitorIndex; x++)
			{
				if ( monitorInfoArray[x].dwFlags == MONITORINFOF_PRIMARY )
				{
					maxwidth = monitorSizeArray[x].cx;
					maxheight = monitorSizeArray[x].cy;
					
					break;
				}
			}
		}
		
		if ( strstr(strVal.c_str(), "@") != NULL )
			i = i+2;

		if (strVal[i] == '-')
		{
			negative = true;
			i++;
		}

		while (i < strVal.length())
		{
			if (strVal[i] >= '0' && strVal[i] <= '9')
			{
				value = (value * 10) + (strVal[i] - '0');
			}
			else
			{
				if (strVal[i] == '%')
					percentage = true;

				break;
			}

			i++;
		}

		if (percentage)
			value = (value * (vertical ? maxheight : maxwidth)) / 100;

		if (negative)
			value = (vertical ? maxheight : maxwidth) - value;
	}
	else
	{
		if ( strstr(strVal.c_str(), "@") != NULL )
			i= i+2;

		if (strVal[i] == '-')
		{
			negative = true;
			i++;
		}

		while (i < strVal.length())
		{
			if (strVal[i] >= '0' && strVal[i] <= '9')
			{
				value = (value * 10) + (strVal[i] - '0');
			}
			else
			{
				if (strVal[i] == '%')
					percentage = true;

				break;
			}

			i++;
		}

		if (percentage)
			value = (value * maxVal) / 100;

		if (negative)
			value = maxVal - value;
	}

	return value;
}

// retrieve a dimension (absolute value or percentage) from step.rc
int GetxRCDimen(const string &uniqueKey, const string &baseName, int defaultVal, int maxVal, HWND box, bool vertical)
{
	string strVal = GetxRCString(uniqueKey, baseName, "");
	return xParseDimen(strVal, defaultVal, maxVal, box, vertical);
}

// enhanced version of PaintDesktop
void PaintDesktopEx(HDC hdcDest, int xDest, int yDest, int cxDest, int cyDest, int xSrc, int ySrc, BOOL updateCache)
{
	if(hdcDesktop == 0 || updateCache)
	{
		if(hdcDesktop == 0 && hdcDest == 0)
			return;

		if(hbmDesktop != 0)
		{
			hbmDesktop = (HBITMAP) SelectObject(hdcDesktop, hbmDesktop);
			DeleteObject(hbmDesktop);
			hbmDesktop = 0;
		}

		HDC hdcScreen = GetDC(0);

		if(hdcDesktop == 0)
		{
			hdcDesktop = CreateCompatibleDC(hdcScreen);
		}
		
		int cxDesktop = GetSystemMetrics(SM_CXSCREEN);
		int cyDesktop = GetSystemMetrics(SM_CYSCREEN);

		hbmDesktop = CreateCompatibleBitmap(hdcScreen, cxDesktop, cyDesktop);
		hbmDesktop = (HBITMAP) SelectObject(hdcDesktop, hbmDesktop);

		RECT r;
		SetRect(&r, 0, 0, cxDesktop, cyDesktop);
		FillRect(hdcDesktop, &r, GetSysColorBrush(COLOR_DESKTOP));

		char wallpaper[MAX_PATH_LENGTH] = { 0 };

		RegQueryStringValue(HKEY_CURRENT_USER,
			"Control Panel\\Desktop", "Wallpaper", wallpaper,
			MAX_PATH_LENGTH, "");

		if(wallpaper[0])
		{
			int wallpaperStyle;
			int tileWallpaper;

			char buffer[32];

			RegQueryStringValue(HKEY_CURRENT_USER, "Control Panel\\Desktop", "WallpaperStyle", buffer, 32, "0");
			wallpaperStyle = atoi(buffer);

			RegQueryStringValue(HKEY_CURRENT_USER, "Control Panel\\Desktop", "TileWallpaper", buffer, 32, "0");
			tileWallpaper = atoi(buffer);

			HBITMAP hbmWallpaper = (HBITMAP) LoadImage(0, wallpaper,
				IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);

			if(hbmWallpaper)
			{
				BITMAP bm;
				GetObject(hbmWallpaper, sizeof(BITMAP), &bm);

				int cxWallpaper = bm.bmWidth;
				int cyWallpaper = bm.bmHeight;

				HDC hdcWallpaper = CreateCompatibleDC(hdcScreen);
				hbmWallpaper = (HBITMAP) SelectObject(hdcWallpaper, hbmWallpaper);

				if(tileWallpaper)
				{
					TileBlt(hdcDesktop,
						0, 0,
						cxDesktop, cyDesktop,
						hdcWallpaper,
						0, 0,
						cxWallpaper, cyWallpaper,
						SRCCOPY);
				}
				else
				{
					if(wallpaperStyle == 2) // stretch
					{
						SetStretchBltMode(hdcDesktop, STRETCH_DELETESCANS);

						StretchBlt(hdcDesktop,
							0, 0,
							cxDesktop, cyDesktop,
							hdcWallpaper,
							0, 0,
							cxWallpaper, cyWallpaper,
							SRCCOPY);
					}
					else
					{
						BitBlt(hdcDesktop,
						(cxDesktop - cxWallpaper) / 2,
						(cyDesktop - cyWallpaper) / 2,
						cxWallpaper, cyWallpaper,
						hdcWallpaper,
						0, 0,
						SRCCOPY);
					}
				}

				hbmWallpaper = (HBITMAP) SelectObject(hdcWallpaper, hbmWallpaper);
				DeleteDC(hdcWallpaper);

				DeleteObject(hbmWallpaper);
			}
		}

		ReleaseDC(0, hdcScreen);
	}

	if(hdcDest)
	{
		BitBlt(hdcDest, xDest, yDest, cxDest, cyDest, hdcDesktop, xSrc, ySrc, SRCCOPY);
	}
}

// tile an image horizontally and vertically
void TileBlt(HDC hdcDest, int xDest, int yDest, int cxDest, int cyDest, HDC hdcSrc, int xSrc, int ySrc, int cxSrc, int cySrc, DWORD rasterOp)
{
	for(int y = 0; y < cyDest; y += cySrc)
	{
		for(int x = 0; x < cxDest; x += cxSrc)
		{
			BitBlt(hdcDest,
				xDest + x,
				yDest + y,
				min(cxDest - x, cxSrc),
				min(cyDest - y, cySrc),
				hdcSrc,
				xSrc,
				ySrc,
				rasterOp);
		}
	}
}

// tile an image horizontally
void HorizontalTileBlt(HDC hdcDest, int xDest, int yDest, int cxDest, int cyDest, HDC hdcSrc, int xSrc, int ySrc, int cxSrc, int cySrc, DWORD rasterOp)
{
	for(int x = 0; x < cxDest; x += cxSrc)
	{
		StretchBlt(hdcDest,
			xDest + x,
			yDest,
			min(cxDest - x, cxSrc),
			cyDest,
			hdcSrc,
			xSrc,
			ySrc,
			min(cxDest - x, cxSrc),
			cySrc,
			rasterOp);
	}
}

// tile an image vertically
void VerticalTileBlt(HDC hdcDest, int xDest, int yDest, int cxDest, int cyDest, HDC hdcSrc, int xSrc, int ySrc, int cxSrc, int cySrc, DWORD rasterOp)
{
	for(int y = 0; y < cyDest; y += cySrc)
	{
		StretchBlt(hdcDest,
			xDest,
			yDest + y,
			cxDest,
			min(cyDest - y, cySrc),
			hdcSrc,
			xSrc,
			ySrc,
			cxSrc,
			min(cyDest - y, cySrc),
			rasterOp);
	}
}

// multi-blt!
void MultiBlt(HDC hdcDest, int xDest, int yDest, int cxDest, int cyDest, HDC hdcSrc, int xSrc, int ySrc, int cxSrc, int cySrc, int cxLeft, int cyTop, int cxRight, int cyBottom, int mode, DWORD rasterOp)
{
	if(cyTop > 0)
	{
		if(cxLeft > 0)
		{
			BitBlt(hdcDest,
				xDest,
				yDest,
				cxLeft,
				cyTop,
				hdcSrc,
				xSrc,
				ySrc,
				rasterOp);
		}

		if(mode == MULTIBLT_STRETCH || mode == MULTIBLT_TILEVERTICAL)
		{
			StretchBlt(hdcDest,
				xDest + cxLeft,
				yDest,
				cxDest - cxLeft - cxRight,
				cyTop,
				hdcSrc,
				xSrc + cxLeft,
				ySrc,
				cxSrc - cxLeft - cxRight,
				cyTop,
				rasterOp);
		}
		else
		{
			HorizontalTileBlt(hdcDest,
				xDest + cxLeft,
				yDest,
				cxDest - cxLeft - cxRight,
				cyTop,
				hdcSrc,
				xSrc + cxLeft,
				ySrc,
				cxSrc - cxLeft - cxRight,
				cyTop,
				rasterOp);
		}

		if(cxRight > 0)
		{
			BitBlt(hdcDest,
				cxDest - cxRight,
				yDest,
				cxRight,
				cyTop,
				hdcSrc,
				cxSrc - cxRight,
				ySrc,
				rasterOp);
		}
	}

	if(cxLeft > 0)
	{
		if(mode == MULTIBLT_STRETCH || mode == MULTIBLT_TILEHORIZONTAL)
		{
			StretchBlt(hdcDest,
				xDest,
				yDest + cyTop,
				cxLeft,
				cyDest - cyTop - cyBottom,
				hdcSrc,
				xSrc,
				ySrc + cyTop,
				cxLeft,
				cySrc - cyTop - cyBottom,
				rasterOp);
		}
		else
		{
			VerticalTileBlt(hdcDest,
				xDest,
				yDest + cyTop,
				cxLeft,
				cyDest - cyTop - cyBottom,
				hdcSrc,
				xSrc,
				ySrc + cyTop,
				cxLeft,
				cySrc - cyTop - cyBottom,
				rasterOp);
		}
	}

	if(mode == MULTIBLT_STRETCH)
	{
		StretchBlt(hdcDest,
			xDest + cxLeft,
			yDest + cyTop,
			cxDest - cxLeft - cxRight,
			cyDest - cyTop - cyBottom,
			hdcSrc,
			xSrc + cxLeft,
			ySrc + cyTop,
			cxSrc - cxLeft - cxRight,
			cySrc - cyTop - cyBottom,
			rasterOp);
	}
	else if(mode == MULTIBLT_TILE)
	{
		TileBlt(hdcDest,
			xDest + cxLeft,
			yDest + cyTop,
			cxDest - cxLeft - cxRight,
			cyDest - cyTop - cyBottom,
			hdcSrc,
			xSrc + cxLeft,
			ySrc + cyTop,
			cxSrc - cxLeft - cxRight,
			cySrc - cyTop - cyBottom,
			rasterOp);
	}
	else if(mode == MULTIBLT_TILEHORIZONTAL)
	{
		HorizontalTileBlt(hdcDest,
			xDest + cxLeft,
			yDest + cyTop,
			cxDest - cxLeft - cxRight,
			cyDest - cyTop - cyBottom,
			hdcSrc,
			xSrc + cxLeft,
			ySrc + cyTop,
			cxSrc - cxLeft - cxRight,
			cySrc - cyTop - cyBottom,
			rasterOp);
	}
	else if(mode == MULTIBLT_TILEVERTICAL)
	{
		VerticalTileBlt(hdcDest,
			xDest + cxLeft,
			yDest + cyTop,
			cxDest - cxLeft - cxRight,
			cyDest - cyTop - cyBottom,
			hdcSrc,
			xSrc + cxLeft,
			ySrc + cyTop,
			cxSrc - cxLeft - cxRight,
			cySrc - cyTop - cyBottom,
			rasterOp);
	}

	if(cxRight > 0)
	{
		if(mode == MULTIBLT_STRETCH || mode == MULTIBLT_TILEHORIZONTAL)
		{
			StretchBlt(hdcDest,
				cxDest - cxRight,
				yDest + cyTop,
				cxRight,
				cyDest - cyTop - cyBottom,
				hdcSrc,
				cxSrc - cxRight,
				ySrc + cyTop,
				cxRight,
				cySrc - cyTop - cyBottom,
				rasterOp);
		}
		else
		{
			VerticalTileBlt(hdcDest,
				cxDest - cxRight,
				yDest + cyTop,
				cxRight,
				cyDest - cyTop - cyBottom,
				hdcSrc,
				cxSrc - cxRight,
				ySrc + cyTop,
				cxRight,
				cySrc - cyTop - cyBottom,
				rasterOp);
		}
	}

	if(cyBottom > 0)
	{
		if(cxLeft > 0)
		{
			BitBlt(hdcDest,
				xDest,
				cyDest - cyBottom,
				cxLeft,
				cyBottom,
				hdcSrc,
				xSrc,
				cySrc - cyBottom,
				rasterOp);
		}

		if(mode == MULTIBLT_STRETCH || mode == MULTIBLT_TILEVERTICAL)
		{
			StretchBlt(hdcDest,
				xDest + cxLeft,
				cyDest - cyBottom,
				cxDest - cxLeft - cxRight,
				cyBottom,
				hdcSrc,
				xSrc + cxLeft,
				cySrc - cyBottom,
				cxSrc - cxLeft - cxRight,
				cyBottom,
				rasterOp);
		}
		else
		{
			HorizontalTileBlt(hdcDest,
				xDest + cxLeft,
				cyDest - cyBottom,
				cxDest - cxLeft - cxRight,
				cyBottom,
				hdcSrc,
				xSrc + cxLeft,
				cySrc - cyBottom,
				cxSrc - cxLeft - cxRight,
				cyBottom,
				rasterOp);
		}

		if(cxRight > 0)
		{
			BitBlt(hdcDest,
				cxDest - cxRight,
				cyDest - cyBottom,
				cxRight,
				cyBottom,
				hdcSrc,
				cxSrc - cxRight,
				cySrc - cyBottom,
				rasterOp);
		}
	}
}

// retrieve a string value from the registry
LPTSTR RegQueryStringValue(HKEY hKey, LPCTSTR pszSubKey, LPCTSTR pszValueName, LPTSTR pszBuffer, int nBufferSize, LPTSTR pszDefaultValue)
{
	DWORD dwDataType;
	DWORD dwDataSize = (DWORD) nBufferSize;
	HKEY hSubKey = NULL;
	LONG lErrorCode;
	
	if(pszSubKey)
	{
		lErrorCode = RegOpenKeyEx(hKey, pszSubKey, 0, KEY_QUERY_VALUE, &hSubKey);
		
		if(lErrorCode != ERROR_SUCCESS)
		{
			return pszBuffer;
		}
		
		hKey = hSubKey;
	}
	
	lErrorCode = RegQueryValueEx(hKey, pszValueName, NULL, &dwDataType,
		(LPBYTE) pszBuffer, &dwDataSize);
	
	if(hSubKey)
		RegCloseKey(hSubKey);
	
	return pszBuffer;
}

string IntToString(int input)
{
	char buffer[32];
	itoa( input, buffer, 10 );
	return (string) buffer;
}
