/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2005 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "WindowUtilities.h"

typedef WINUSERAPI BOOL (WINAPI *FUNC_BOOL__DWORD)(DWORD);
#ifndef ASFW_ANY
	#define ASFW_ANY ((DWORD)-1)
#endif

extern FUNC_BOOL__DWORD pAllowSetForegroundWindow;

bool IsActive(HWND hWindow)
{
	HWND hOwner = GetOwnerWindow(GetForegroundWindow());

	if ( hOwner )
		return ( hOwner == hWindow );
	else
		return ( GetForegroundWindow() == hWindow);
}

int GetState(HWND hWindow)
{
	if ( IsIconic( GetRealWindow(hWindow) ))
        return STATE_MINIMIZED;
	else if (IsActive(hWindow))
        return STATE_SELECTED;
	else
		return STATE_NORMAL;
}

void OnTopToggle(HWND hWindow)
{
	if (!IsOnTop(hWindow))
	{
		SetOnTop(hWindow);
	}
	else
	{
		SetNotOnTop(hWindow);
	}
}

bool IsOnTop(HWND hWindow)
{
	LONG lWindowLong = GetWindowLong(hWindow, GWL_EXSTYLE);
	if( lWindowLong  )
	{
		return ((lWindowLong& WS_EX_TOPMOST) == WS_EX_TOPMOST );
	}
	else
	{
		return false;
	}
}

void SetOnTop(HWND hWindow)
{
	EndDeferWindowPos(DeferWindowPos(BeginDeferWindowPos(1), hWindow, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE));
}

void SetNotOnTop(HWND hWindow)
{
	EndDeferWindowPos(DeferWindowPos(BeginDeferWindowPos(1), hWindow, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE));
}

void MinimizeWindow(HWND hWindow)
{
	ShowWindowAsync( hWindow, SW_MINIMIZE );
}

void MaximizeWindow(HWND hWindow)
{
	ShowWindowAsync( hWindow, SW_MAXIMIZE );
}

void RestoreWindow(HWND hWindow)
{
	if ( pAllowSetForegroundWindow != NULL )
		pAllowSetForegroundWindow(ASFW_ANY);

	SetForegroundWindow( hWindow );

	ShowWindowAsync( hWindow, SW_RESTORE );
}

void SelectWindow(HWND hWindow, BOOL restore)
{
	if ( pAllowSetForegroundWindow != NULL )
		pAllowSetForegroundWindow(ASFW_ANY);

	SetForegroundWindow( hWindow );

	if ( restore )
		ShowWindowAsync( hWindow, SW_RESTORE );
}

void Close(HWND hWindow)
{
	PostMessage(hWindow, WM_CLOSE,0,0);
}

HWND GetOwnerWindow(HWND hWindow)
{
	return GetWindow(hWindow, GW_OWNER);
}

HWND GetRealWindow(HWND hWindow)
{
	HWND hOwner = GetOwnerWindow(hWindow);

	if (hOwner)
	{
		return hOwner;
	}
	else
	{
		return hWindow;
	}
}
