/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2005 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "StdAfx.h"
#include "Taskbar.h"

#include "VerInfo.h"

typedef list<Taskbar *> TaskbarList;
typedef list<Taskbar *>::iterator TaskbarListIterator;

// Bang commands
void BangAlwaysOnTop(HWND hwndCaller, LPCSTR pszArgs);
void BangCreate(HWND hwndCaller, LPCSTR pszArgs);
void BangDestroy(HWND hwndCaller, LPCSTR pszArgs);
void BangDisplay(HWND hwndCaller, LPCSTR pszArgs);
void BangHide(HWND hwndCaller, LPCSTR pszArgs);
void BangLSBoxHook(HWND hwndCaller, LPCSTR pszArgs);
void BangMove(HWND hwndCaller, LPCSTR pszArgs);
void BangMoveBy(HWND hwndCaller, LPCSTR pszArgs);
void BangRefresh(HWND hwndCaller, LPCSTR pszArgs);
void BangReposition(HWND hwndCaller, LPCSTR pszArgs);
void BangRepositionBy(HWND hwndCaller, LPCSTR pszArgs);
void BangResize(HWND hwndCaller, LPCSTR pszArgs);
void BangResizeBy(HWND hwndCaller, LPCSTR pszArgs);
void BangSetAlpha(HWND hwndCaller, LPCSTR pszArgs);
void BangShow(HWND hwndCaller, LPCSTR pszArgs);
void BangSwitch(HWND hwndCaller, LPCSTR pszArgs);
void BangToggle(HWND hwndCaller, LPCSTR pszArgs);

HINSTANCE hInstance;
HWND messageHandler;

int lsMessages[] = {
	LM_GETREVID,
	LM_REFRESH,
	0
};

int screenWidth;
int screenHeight;
int minscreenX;
int	minscreenY;
int maxscreenWidth;
int	maxscreenHeight;

int realMonitors = 1;
int monitorIndex = 0;
MONITORINFO monitorInfoArray[3];
SIZE monitorSizeArray[3];

bool singleTaskbar = true;

int bitsperpixel = atoi( QueryPixelDepth().c_str() );

TaskbarList taskbarList;

Taskbar *lookupTaskbar( const string &name = "" );

Taskbar *lookupTaskbar( const string &name )
{
	if (!name.empty())
	{
		for(TaskbarListIterator tblit = taskbarList.begin(); tblit != taskbarList.end(); ++tblit)
		{
			if(stricmp(name.c_str(), (*tblit)->getName().c_str()) == NULL)
				return *tblit;
		}
		return NULL;
	}
	else if (taskbarList.size() != 0)
	{
		return *taskbarList.begin();
	}
	else
	{
		return NULL;
	}
}

#define LM_UPDATEBG (WM_USER + 1)

LRESULT WINAPI MessageHandlerProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch(message)
	{
		case LM_GETREVID:
		{
			UINT uLength;
			StringCchPrintf((char*)lParam, 64, "%s.dll: %s", V_NAME, V_VERSION);
			
			if (SUCCEEDED(StringCchLength((char*)lParam, 64, &uLength)))
				return uLength;

			lParam = NULL;
			return 0;
		}

		case LM_REFRESH:
		{
			for(TaskbarListIterator tblit = taskbarList.begin(); tblit != taskbarList.end(); ++tblit)
			{
				(*tblit)->reconfigure(true);
				(*tblit)->CreateLayout(1);
			}

			return 0;
		}

		case LM_UPDATEBG:
		{
			PaintDesktopEx(0, 0, 0, 0, 0, 0, 0, TRUE);

			for(TaskbarListIterator tblit = taskbarList.begin(); tblit != taskbarList.end(); ++tblit)
			{
				(*tblit)->repaint();
			}

			return 0;
		}


		case WM_DISPLAYCHANGE:
		case WM_SETTINGCHANGE:
		{
			PostMessage(hWnd, LM_UPDATEBG, 0, 0);

			return 0;
		}
	}

	return DefWindowProc(hWnd, message, wParam, lParam);
}

// Why are we advoiding the crt startup code? The start up code is not large
// enough to cause bloat (except possibly a long system call, and I suspect that
// would need done eventually anyway) (default _DllMainCRTStartup is in
// (MSVCDIR\crt\src\dllcrt0.c) The elimination of the call to _CRT_INIT is 
// also causing warnings with MSVC7
BOOL __stdcall _DllMainCRTStartup(HINSTANCE hInst, DWORD fdwReason, LPVOID lpvRes)
{
	// We don't need thread notifications for what we're doing.  Thus, get
	// rid of them, thereby eliminating some of the overhead of this DLL
	DisableThreadLibraryCalls(hInst);
	return TRUE;
}

int initModuleEx(HWND hParent, HINSTANCE hInstance, const char *lsPath)
{
	WNDCLASSEX wc;

	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = CS_GLOBALCLASS;
	wc.lpfnWndProc = Taskbar::windowProcedure;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = sizeof(Taskbar *);
	wc.hInstance = hInstance;
	wc.hbrBackground = 0;
	wc.hCursor = LoadCursor(0, IDC_ARROW);
	wc.hIcon = 0;
	wc.lpszMenuName = 0;
	wc.lpszClassName = "xTaskbar";
	wc.hIconSm = 0;

	RegisterClassEx(&wc);

	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = CS_GLOBALCLASS;
	wc.lpfnWndProc = MessageHandlerProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hbrBackground = 0;
	wc.hCursor = 0;
	wc.hIcon = 0;
	wc.lpszMenuName = 0;
	wc.lpszClassName = "xTaskbarMessageHandler";
	wc.hIconSm = 0;

	RegisterClassEx(&wc);

	messageHandler = CreateWindowEx(WS_EX_TOOLWINDOW,
		"xTaskbarMessageHandler",
		0,
		WS_POPUP,
		0, 0, 0, 0, 
		0,
		0,
		hInstance,
		0);

	if (!messageHandler)
		return 1;
	
	SendMessage(GetLitestepWnd(),
		LM_REGISTERMESSAGE,
		(WPARAM) messageHandler,
		(LPARAM) lsMessages);

	::hInstance = hInstance;

	if ( !IsOS( OS_95 ) && !IsOS( OS_NT4 ) )
	{
		getMonitorInfo();
	
		if ( realMonitors > 1 )
		{
			minscreenX = GetSystemMetrics(SM_XVIRTUALSCREEN);
			minscreenY = GetSystemMetrics(SM_YVIRTUALSCREEN);
			maxscreenWidth = GetSystemMetrics(SM_CXVIRTUALSCREEN);
			maxscreenHeight = GetSystemMetrics(SM_CYVIRTUALSCREEN);
			screenWidth = GetSystemMetrics(SM_CXSCREEN);
			screenHeight = GetSystemMetrics(SM_CYSCREEN);
		}
		else
		{
			minscreenX = 0;
			minscreenY = 0;
			screenWidth = maxscreenWidth = GetSystemMetrics(SM_CXSCREEN);
			screenHeight = maxscreenHeight = GetSystemMetrics(SM_CYSCREEN);
		}
	}
	else
	{
		minscreenX = 0;
		minscreenY = 0;
		screenWidth = maxscreenWidth = GetSystemMetrics(SM_CXSCREEN);
		screenHeight = maxscreenHeight = GetSystemMetrics(SM_CYSCREEN);
	}

	AddBangCommand("!xTaskbarAlwaysOnTop", BangAlwaysOnTop);
	AddBangCommand("!xTaskbarCreate", BangCreate);
	AddBangCommand("!xTaskbarDestroy", BangDestroy);
	AddBangCommand("!xTaskbarDisplay", BangDisplay);
	AddBangCommand("!xTaskbarHide", BangHide);
	AddBangCommand("!xTaskbarLSBoxHook", BangLSBoxHook);
	AddBangCommand("!xTaskbarMove", BangMove);
	AddBangCommand("!xTaskbarMoveBy", BangMoveBy);
	AddBangCommand("!xTaskbarRefresh", BangRefresh);
	AddBangCommand("!xTaskbarReposition", BangReposition);
	AddBangCommand("!xTaskbarRepositionBy", BangRepositionBy);
	AddBangCommand("!xTaskbarResize", BangResize);
	AddBangCommand("!xTaskbarResizeBy", BangResizeBy);
	if(IsOS(OS_2KXP))
		AddBangCommand("!xTaskbarSetAlpha", BangSetAlpha);
	AddBangCommand("!xTaskbarShow", BangShow);
	AddBangCommand("!xTaskbarSwitch", BangSwitch);
	AddBangCommand("!xTaskbarToggle", BangToggle);

	StringList taskbarNames;

	FILE * f = LCOpen(NULL);
	if(f != NULL)
	{
		char lineBuffer[MAX_LINE_LENGTH];

		while(LCReadNextConfig(f, "*xtaskbar", lineBuffer, MAX_LINE_LENGTH))
		{
			char temp[64], name[64];
			char *buffers[] = {temp, name};
			
			if(LCTokenize(lineBuffer, buffers, 2, NULL) == 2)
			{
				taskbarNames.insert(taskbarNames.end(), name);
			}
		}

		LCClose(f);
	}

	taskbarList.clear();

	if ( !taskbarNames.empty() )
	{
		singleTaskbar = false;

		for(StringListIterator sit = taskbarNames.begin(); sit != taskbarNames.end(); ++sit)
		{
			if ( !GetxRCBool((*sit).c_str(), "LoadInBox") )
			{
				Taskbar *taskbar = new Taskbar(*sit);
			
				taskbarList.insert(taskbarList.end(), taskbar);
				
				taskbar->load(hInstance);
			}
		}
	}
	else if ( !GetxRCBool("xTaskbar", "LoadInBox") )
	{
		Taskbar *taskbar = new Taskbar("xTaskbar");
		
		taskbarList.insert(taskbarList.end(), taskbar);
			
		taskbar->load(hInstance);
	}

	return 0;
}

extern HDC hdcDesktop;
extern HBITMAP hbmDesktop;

void quitModule(HINSTANCE hInstance)
{
	RemoveBangCommand("!xTaskbarAlwaysOnTop");
	RemoveBangCommand("!xTaskbarCreate");
	RemoveBangCommand("!xTaskbarDestroy");
	RemoveBangCommand("!xTaskbarDisplay");
	RemoveBangCommand("!xTaskbarHide");
	RemoveBangCommand("!xTaskbarLSBoxHook");
	RemoveBangCommand("!xTaskbarMove");
	RemoveBangCommand("!xTaskbarMoveBy");
	RemoveBangCommand("!xTaskbarRefresh");
	RemoveBangCommand("!xTaskbarReposition");
	RemoveBangCommand("!xTaskbarRepositionBy");
	RemoveBangCommand("!xTaskbarResize");
	RemoveBangCommand("!xTaskbarResizeBy");
	if(IsOS(OS_2KXP))
		RemoveBangCommand("!xTaskbarSetAlpha");
	RemoveBangCommand("!xTaskbarShow");
	RemoveBangCommand("!xTaskbarSwitch");
	RemoveBangCommand("!xTaskbarToggle");

	for(TaskbarListIterator tblit = taskbarList.begin(); tblit != taskbarList.end(); ++tblit)
	{
		(*tblit)->hide();
		delete *tblit;
	}

	taskbarList.clear();

	SendMessage(GetLitestepWnd(),
		LM_UNREGISTERMESSAGE,
		(WPARAM) messageHandler,
		(LPARAM) lsMessages);

	DestroyWindow(messageHandler);

	UnregisterClass("xTaskbar", hInstance);
	UnregisterClass("xTaskbarMessageHandler", hInstance);

	if (hbmDesktop != 0 || hdcDesktop != 0)
	{
		hbmDesktop = (HBITMAP) SelectObject(hdcDesktop, hbmDesktop);
		DeleteDC(hdcDesktop);
		DeleteObject(hbmDesktop);
	}
}

// !xTaskbarCreate <taskbar>
void BangCreate(HWND hwndCaller, LPCSTR pszArgs)
{
	string args = pszArgs;
	if ( args.empty() )
		args = "xTaskbar";

	Taskbar *taskbar = new Taskbar(args);
	taskbarList.insert(taskbarList.end(), taskbar);
	taskbar->load(hInstance);
}

// !xTaskbarDestroy <taskbar>
void BangDestroy(HWND hwndCaller, LPCSTR pszArgs)
{
	Taskbar *taskbar = lookupTaskbar(pszArgs);

	if(taskbar)
	{
		taskbar->hide();
		taskbarList.remove(taskbar);
		delete taskbar;
	}
}

// !xTaskbarAlwaysOnTop <on|true, off|false, toggle|switch>
void BangAlwaysOnTop(HWND hwndCaller, LPCSTR pszArgs)
{
	char name[64], option[16];
	char *buffers[] = {name, option};

	int numTokens = LCTokenize(pszArgs, buffers, 2, NULL);

	if (numTokens == 1)
	{
		Taskbar *taskbar = lookupTaskbar();

		if(taskbar)
		{
			if (stricmp(name, "false") == NULL || stricmp(name, "off") == NULL)
				taskbar->setAlwaysOnTop(false);
			else if (stricmp(name, "toggle") == NULL || stricmp(name, "switch") == NULL)
				taskbar->setAlwaysOnTop(!taskbar->isAlwaysOnTop());
			else
				taskbar->setAlwaysOnTop(true);
		}
	}
	else if (numTokens == 2)
	{
		Taskbar *taskbar = lookupTaskbar(name);

		if(taskbar)
		{
			if (stricmp(option, "false") == NULL || stricmp(option, "off") == NULL)
				taskbar->setAlwaysOnTop(false);
			else if (stricmp(option, "toggle") == NULL || stricmp(option, "switch") == NULL)
				taskbar->setAlwaysOnTop(!taskbar->isAlwaysOnTop());
			else
				taskbar->setAlwaysOnTop(true);
		}
	}
}

// !xTaskbarShow
void BangShow(HWND hwndCaller, LPCSTR pszArgs)
{
	Taskbar *taskbar = lookupTaskbar(pszArgs);

	if(taskbar)
	{
		if ( taskbar->hideIfEmpty && taskbar->backupExportedButtonCount == 0 )
			return;
		
		taskbar->show();
	}
}

// !xTaskbarHide
void BangHide(HWND hwndCaller, LPCSTR pszArgs)
{
	Taskbar *taskbar = lookupTaskbar(pszArgs);

	if(taskbar)
	{
		taskbar->hide();
	}
}

// !xTaskbarToggle
void BangToggle(HWND hwndCaller, LPCSTR pszArgs)
{
	Taskbar *taskbar = lookupTaskbar(pszArgs);

	if(taskbar)
	{
		if(taskbar->isVisible())
		{
			taskbar->hide();
		}
		else
		{
			if ( taskbar->hideIfEmpty && taskbar->backupExportedButtonCount == 0 )
				return;
			
			taskbar->show();
		}
	}
}

// !xTaskbarMove <x> <y> [<steps> <time>]
void BangMove(HWND hwndCaller, LPCSTR pszArgs)
{
	int x = 0;
	int y = 0;
	char name[64], A[16], B[16], C[16], D[16];
	char *buffers[] = {name, A, B, C, D};

	int tokens = LCTokenize(pszArgs, buffers, 5, NULL);

	if (tokens == 2 || tokens == 4)
	{
		Taskbar *taskbar = lookupTaskbar();

		if(taskbar)
		{
			if (taskbar->getBox() == NULL)
			{
				x = xParseCoord(name, 0, screenWidth, NULL, false, taskbar->isAlwaysOnTop());
				y = xParseCoord(A, 0, screenHeight, NULL, true, taskbar->isAlwaysOnTop());
			}
			else
			{
				RECT rc;
				GetClientRect(taskbar->getBox(), &rc);
				x = xParseCoord(name, 0, rc.right, taskbar->getBox());
				y = xParseCoord(A, 0, rc.bottom, taskbar->getBox());
			}

			if ( tokens == 4 )
			{
				int step = atoi( B );
				int time = atoi( C );

				taskbar->reposition(1, x, y, 0, 0, step, time);
			}
			else
			{
				taskbar->reposition(1, x, y, 0, 0);
			}
		}
	}
	else if (tokens == 3 || tokens == 5)
	{
		Taskbar *taskbar = lookupTaskbar(name);

		if(taskbar)
		{
			if (taskbar->getBox() == NULL)
			{
				x = xParseCoord(A, 0, screenWidth, NULL, false, taskbar->isAlwaysOnTop());
				y = xParseCoord(B, 0, screenHeight, NULL, true, taskbar->isAlwaysOnTop());
			}
			else
			{
				RECT rc;
				GetClientRect(taskbar->getBox(), &rc);
				x = xParseCoord(A, 0, rc.right, taskbar->getBox());
				y = xParseCoord(B, 0, rc.bottom, taskbar->getBox());
			}

			if ( tokens == 5 )
			{
				int step = atoi( C );
				int time = atoi( D );

				taskbar->reposition(1, x, y, 0, 0, step, time);
			}
			else
			{
				taskbar->reposition(1, x, y, 0, 0);
			}

		}
	}
}

// !xTaskbarMoveBy <x> <y> [<steps> <time>]
void BangMoveBy(HWND hwndCaller, LPCSTR pszArgs)
{
	int x = 0;
	int y = 0;
	char name[64], A[16], B[16], C[16], D[16];
	char *buffers[] = {name, A, B, C, D};

	int tokens = LCTokenize(pszArgs, buffers, 5, NULL);

	if (tokens == 2 || tokens == 4)
	{
		Taskbar *taskbar = lookupTaskbar();

		if(taskbar)
		{
			x = atoi( name );
			y = atoi( A );
		
			if ( tokens == 4 )
			{
				int step = atoi( B );
				int time = atoi( C );

				taskbar->reposition(1, x, y, 0, 0, step, time, true);
			}
			else
			{
				taskbar->reposition(1, x, y, 0, 0, 0, 0, true);
			}
		}
	}
	else if (tokens == 3 || tokens == 5)
	{
		Taskbar *taskbar = lookupTaskbar(name);

		if(taskbar)
		{
			x = atoi( A );
			y = atoi( B );
		
			if ( tokens == 5 )
			{
				int step = atoi( C );
				int time = atoi( D );

				taskbar->reposition(1, x, y, 0, 0, step, time, true);
			}
			else
			{
				taskbar->reposition(1, x, y, 0, 0, 0, 0, true);
			}
		}
	}
}

// !xTaskbarReposition <x> <y> <width> <height> [<steps>] [<time>]
void BangReposition(HWND hwndCaller, LPCSTR pszArgs)
{
	int x = 0;
	int y = 0;
	int w = 64;
	int h = 64;
	char name[64], A[16], B[16], C[16], D[16], E[16], F[16];
	char *buffers[] = {name, A, B, C, D, E, F};

	int tokens = LCTokenize(pszArgs, buffers, 7, NULL);

	if (tokens == 4 || tokens == 6)
	{
		Taskbar *taskbar = lookupTaskbar();

		if(taskbar)
		{
			if (taskbar->getBox() == NULL)
			{
				x = xParseCoord(name, 0, screenWidth, NULL, false, taskbar->isAlwaysOnTop());
				y = xParseCoord(A, 0, screenHeight, NULL, true, taskbar->isAlwaysOnTop());
				w = xParseDimen(B, 0, screenWidth, NULL, false);
				h = xParseDimen(C, 0, screenHeight, NULL, true);
			}
			else
			{
				RECT rc;
				GetClientRect(taskbar->getBox(), &rc);
				x = xParseCoord(name, 0, rc.right, taskbar->getBox());
				y = xParseCoord(A, 0, rc.bottom, taskbar->getBox());
				w = xParseDimen(B, 0, rc.right, taskbar->getBox());
				h = xParseDimen(C, 0, rc.bottom, taskbar->getBox());
			}

			if ( tokens == 6 )
			{
				int step = atoi( D );
				int time = atoi( E );

				taskbar->reposition(3, x, y, w, h, step, time);
			}
			else
			{
				taskbar->reposition(3, x, y, w, h);
			}
		}
	}
	else if (tokens == 5 || tokens == 7)
	{
		Taskbar *taskbar = lookupTaskbar(name);

		if(taskbar)
		{
			if (taskbar->getBox() == NULL)
			{
				x = xParseCoord(A, 0, screenWidth, NULL, false, taskbar->isAlwaysOnTop());
				y = xParseCoord(B, 0, screenHeight, NULL, true, taskbar->isAlwaysOnTop());
				w = xParseDimen(C, 0, screenWidth, NULL, false);
				h = xParseDimen(D, 0, screenHeight, NULL, true);
			}
			else
			{
				RECT rc;
				GetClientRect(taskbar->getBox(), &rc);
				x = xParseCoord(A, 0, rc.right, taskbar->getBox());
				y = xParseCoord(B, 0, rc.bottom, taskbar->getBox());
				w = xParseDimen(C, 0, rc.right, taskbar->getBox());
				h = xParseDimen(D, 0, rc.bottom, taskbar->getBox());
			}

			if ( tokens == 7 )
			{
				int step = atoi( E );
				int time = atoi( F );

				taskbar->reposition(3, x, y, w, h, step, time);
			}
			else
			{
				taskbar->reposition(3, x, y, w, h);
			}
		}
	}
}

// !xTaskbarRepositionBy <cx> <cy> <cwidth> <cheight> [<steps>] [<time>]
void BangRepositionBy(HWND hwndCaller, LPCSTR pszArgs)
{
	int x = 0;
	int y = 0;
	int w = 64;
	int h = 64;
	char name[64], A[16], B[16], C[16], D[16], E[16], F[16];
	char *buffers[] = {name, A, B, C, D, E, F};

	int tokens = LCTokenize(pszArgs, buffers, 7, NULL);

	if (tokens == 4 || tokens == 6)
	{
		Taskbar *taskbar = lookupTaskbar();

		if(taskbar)
		{
			x = atoi( name );
			y = atoi( A );
			w = atoi( B );
			h = atoi( C );
			
			if ( tokens == 6 )
			{
				int step = atoi( D );
				int time = atoi( E );

				taskbar->reposition(3, x, y, w, h, step, time, true);
			}
			else
			{
				taskbar->reposition(3, x, y, w, h, 0, 0, true);
			}
		}
	}
	else if (tokens == 5 || tokens == 7)
	{
		Taskbar *taskbar = lookupTaskbar(name);

		if(taskbar)
		{
			x = atoi( A );
			y = atoi( B );
			w = atoi( C );
			h = atoi( D );
		
			if ( tokens == 7 )
			{
				int step = atoi( E );
				int time = atoi( F );

				taskbar->reposition(3, x, y, w, h, step, time, true);
			}
			else
			{
				taskbar->reposition(3, x, y, w, h, 0, 0, true);
			}
		}
	}
}


// !xTaskbarResize <width> <height> [<steps> <time>]
void BangResize(HWND hwndCaller, LPCSTR pszArgs)
{
	int w = 64;
	int h = 64;
	char name[64], A[16], B[16], C[16], D[16];
	char *buffers[] = {name, A, B, C, D};

	int tokens = LCTokenize(pszArgs, buffers, 5, NULL);
	
	if (tokens == 2 || tokens == 4)
	{
		Taskbar *taskbar = lookupTaskbar();

		if(taskbar)
		{
			if (taskbar->getBox() == NULL)
			{
				w = xParseDimen(name, 0, screenWidth, NULL, false);
				h = xParseDimen(A, 0, screenHeight, NULL, true);
			}
			else
			{
				RECT rc;
				GetClientRect(taskbar->getBox(), &rc);
				w = xParseDimen(name, 0, rc.right, taskbar->getBox());
				h = xParseDimen(A, 0, rc.bottom, taskbar->getBox());
			}

			if ( tokens == 4 )
			{
				int step = atoi( B );
				int time = atoi( C );

				taskbar->reposition(2, 0, 0, w, h, step, time);
			}
			else
			{
				taskbar->reposition(2, 0, 0, w, h);
			}
		}
	}
	else if (tokens == 3 || tokens == 5)
	{
		Taskbar *taskbar = lookupTaskbar(name);

		if(taskbar)
		{
			if (taskbar->getBox() == NULL)
			{
				w = xParseDimen(A, 0, screenWidth, NULL, false);
				h = xParseDimen(B, 0, screenHeight, NULL, true);
			}
			else
			{
				RECT rc;
				GetClientRect(taskbar->getBox(), &rc);
				w = xParseDimen(A, 0, rc.right, taskbar->getBox());
				h = xParseDimen(B, 0, rc.bottom, taskbar->getBox());
			}

			if ( tokens == 5 )
			{
				int step = atoi( C );
				int time = atoi( D );

				taskbar->reposition(2, 0, 0, w, h, step, time);
			}
			else
			{
				taskbar->reposition(2, 0, 0, w, h);
			}
		}
	}
}

// !xTaskbarResizeBy <cx> <cy> [<steps> <time>]
void BangResizeBy(HWND hwndCaller, LPCSTR pszArgs)
{
	int w = 64;
	int h = 64;
	char name[64], A[16], B[16], C[16], D[16];
	char *buffers[] = {name, A, B, C, D};

	int tokens = LCTokenize(pszArgs, buffers, 5, NULL);
	
	if (tokens == 2 || tokens == 4)
	{
		Taskbar *taskbar = lookupTaskbar();

		if(taskbar)
		{
			w = atoi( name );
			h = atoi( A );
			
			if ( tokens == 4 )
			{
				int step = atoi( B );
				int time = atoi( C );

				taskbar->reposition(2, 0, 0, w, h, step, time, true);
			}
			else
			{
				taskbar->reposition(2, 0, 0, w, h, 0, 0, true);
			}
		}
	}
	else if (tokens == 3 || tokens == 5)
	{
		Taskbar *taskbar = lookupTaskbar(name);

		if(taskbar)
		{
			w = atoi( A );
			h = atoi( B );
		
			if ( tokens == 5 )
			{
				int step = atoi( C );
				int time = atoi( D );

				taskbar->reposition(2, 0, 0, w, h, step, time, true);
			}
			else
			{
				taskbar->reposition(2, 0, 0, w, h, 0, 0, true);
			}
		}
	}
}

// !xTaskbarLSBoxHook <taskbarname>
void BangLSBoxHook(HWND hwndCaller, LPCSTR pszArgs)
{
	char name[64] = {0};
	char rest[MAX_PATH_LENGTH] = {0};
	char *buffers[] = {name};

	LCTokenize(pszArgs, buffers, 1, rest);

	const char *handle = strrchr(pszArgs,' ');

	HWND hwndBox = (HWND)atoi(handle+1);

	string tmpname = name;

	Taskbar *taskbar = lookupTaskbar(tmpname);

	if (taskbar == NULL)
	{
		taskbar = new Taskbar(name);
		
		taskbarList.insert(taskbarList.end(), taskbar);
				
		taskbar->load(hInstance, hwndBox);
	}
	else
	{
		MessageBox(NULL, ("Use the setting\n\"" + tmpname + "LoadInBox\",\nif you want to hook this xTaskbar in another Module!").c_str(), "User Information", MB_OK | MB_SETFOREGROUND | MB_ICONEXCLAMATION);
	}
}

// !xTaskbarSetAlpha <value>
void BangSetAlpha(HWND hwndCaller, LPCSTR pszArgs)
{	
	char name[64], value[16], step[16], delay[16];
	char *buffers[] = {name, value, step, delay};

	int tokens = LCTokenize(pszArgs, buffers, 4, NULL);

	if (tokens == 1 || tokens == 3)
	{
		Taskbar *taskbar = lookupTaskbar();
		
		if(taskbar)
		{	
			if ( tokens == 1 )
			{		
				taskbar->setAlpha(atoi( name ));	
			}
			else if ( tokens == 3 )
			{
				taskbar->setAlphaFade(atoi( name ), atoi( value ), atoi( step ));
			}
		}
	}
	else if (tokens == 2 || tokens == 4)
	{
		Taskbar *taskbar = lookupTaskbar(name);
		
		if(taskbar)
		{
			if ( tokens == 2 )
			{		
				taskbar->setAlpha(atoi( value ));	
			}
			else if ( tokens == 4 )
			{
				taskbar->setAlphaFade(atoi( value ), atoi( step ), atoi( delay ));
			}
		}
	}

}

// !xTaskbarSwitch <next|prev>
void BangSwitch(HWND hwndCaller, LPCSTR pszArgs)
{
	char name[64], value[32];
	char *buffers[] = {name, value};

	int tokens = LCTokenize(pszArgs, buffers, 2, NULL);

	if (tokens == 1)
	{
		Taskbar *taskbar = lookupTaskbar();

		if(taskbar)
		{
			taskbar->SwitchToButton( name );
		}
	}
	else if (tokens == 2)
	{
		Taskbar *taskbar = lookupTaskbar(name);

		if(taskbar)
		{
			taskbar->SwitchToButton( value );
		}
	}
}

// !xTaskbarDisplay <value>
void BangDisplay(HWND hwndCaller, LPCSTR pszArgs)
{
	NameValuePair displayValues[] = {
			{ "flashing", -2 },
			{ "all", -1 },
			{ "normal+active+minimized", -1 },
			{ "normal", 0 },
			{ "active", 1 },
			{ "minimized", 2 },
			{ "active+normal", 3 },
			{ "normal+active", 3 },
			{ "minimized+normal", 4 },
			{ "normal+minimized", 4 },
			{ "active+minimized", 5 },
			{ "minimized+active", 5 },
			{ 0, 0 }
			};

	const NameValuePair *current = displayValues;

	int display = -1;

	char name[64], value[64];
	char *buffers[] = {name, value};

	int tokens = LCTokenize(pszArgs, buffers, 2, NULL);

	if (tokens == 1)
	{
		while(current->name)
		{
			if(stricmp(current->name, name) == 0)
			{
				display = current->value;
				break;
			}

			current++;
		}

		Taskbar *taskbar = lookupTaskbar();
		
		if(taskbar)
		{
			taskbar->displayButtonType = display;
			taskbar->CreateLayout(1);
		}
	}
	else if (tokens == 2)
	{
		while(current->name)
		{
			if( stricmp(current->name, value) == 0)
			{
				display = current->value;
				break;
			}

			current++;
		}

		Taskbar *taskbar = lookupTaskbar(name);
		
		if(taskbar)
		{
			taskbar->displayButtonType = display;
			taskbar->CreateLayout(1);
		}
	}
}

// !xTaskbarRefresh <setting> <newsettings>
void BangRefresh(HWND hwndCaller, LPCSTR pszArgs)
{
	char name[MAX_PATH_LENGTH], setting[MAX_PATH_LENGTH], value[MAX_PATH_LENGTH];
	char *buffers[] = {name, setting};

	LCTokenize(pszArgs, buffers, 2, value);

	Taskbar *taskbar = lookupTaskbar(name);

	if (taskbar)
	{
		if ( taskbar->isLayoutLocked() )
			return;

		PrefixedLSSetVariable( (string)setting, "", (string)value );
		taskbar->reconfigure(true);
	}
	else
	{
		taskbar = lookupTaskbar();

		if (taskbar)
		{
			if ( taskbar->isLayoutLocked() )
				return;

			PrefixedLSSetVariable( (string)name, "", (string)setting );
			taskbar->reconfigure(true);
		}
	}
}